<?php
class WEAPP_Live_Room_Module extends WPJAM_Module{
	public function parse($args=[]){
		$data		= parent::parse($args);
		$options	= $this->get_live_statuses();
		$statuses	= $data['setting']['statuses'] ?: array_keys($options);
		$statuses	= array_map('intval', $statuses);
		$texts		= [];
		$number		= $data['setting']['number'] ?: 10;

		if($number > 10 || $number < 0){
			$number	= 10;
		}

		foreach ($statuses as $status) {
			if(isset($options[$status])){
				$texts[]	= ['status'=>$status, 'text'=>$options[$status]];
			}
		}

		$data['setting']['statuses']	= $statuses;
		$data['setting']['texts']		= $texts;
		$data['setting']['number']		= intval($number);

		$live_rooms	= [];
		$response	= weapp_get_live_rooms();

		if($response && !is_wp_error($response)){
			if($room_info = $response['room_info']){

				$numbers 	= array_fill_keys($statuses, $number);

				foreach($room_info as $live_room){
					$live_status	= $live_room['live_status'];

					if(in_array($live_status, $statuses) && $numbers[$live_status]){
						$live_room['start_time']	= $live_room['start_time'] ? get_date_from_gmt(date('Y-m-d H:i:s', $live_room['start_time'])) : '';
						$live_room['end_time']		= $live_room['end_time'] ? get_date_from_gmt(date('Y-m-d H:i:s', $live_room['end_time'])) : '';
						$live_room['status']		= $live_room['live_status'];
						$live_room['id']			= $live_room['roomid'];

						unset($live_room['live_status']);
						unset($live_room['roomid']);

						$live_rooms[]	= $live_room;

						$numbers[$live_status] --;
					}
				}
			}
		}

		$data['content']	= $live_rooms;
		
		return $data;
	}

	public function render_content(){
		$content	= '';
		$options	= $this->get_live_statuses();
		$number		= $this->setting['number'] ?: 10;
		$statuses	= $this->setting['statuses'] ?: array_keys($options);
		$response	= weapp_get_live_rooms();

		if($response && !is_wp_error($response)){
			$live_rooms	= [];

			if($room_info = $response['room_info']){
				$i	= 0;

				foreach($room_info as $live_room){
					if(in_array($live_room['live_status'], $statuses)){
						$live_rooms[]	= '<strong>'.$options[$live_room['live_status']].'</strong> | '.$live_room['name'];
					}

					$i++;

					if($i >= $number){
						break;
					}
				}
			}

			if($live_rooms){
				$content	= implode("<br />", $live_rooms);
			}
		}
		
		return $content;
	}

	public static function get_fields($action_key='', $args=[]){
		if($action_key == 'setting'){
			$fields['statuses']	= ['title'=>'直播状态',	'type'=>'checkbox',	'options'=>static::get_live_statuses()];
			$fields['number']	= ['title'=>'显示数量',	'type'=>'number',	'class'=>'',	'max'=>10];
		}elseif($action_key == 'create' || $action_key == 'edit'){
			$fields['value'] 	= ['title'=>' ', 'type'=>'hidden',	'name'=>'content[value]','value' =>1];
		}

		return $fields;
	}

	public static function get_live_statuses(){
		return [101=>'正在直播',	102=>'直播预告', 103=>'直播回放'];
	}
}

class_alias('WEAPP_Live_Room_Module', 'WPJAM_ModuleLiveRoom');