<?php
class WEAPP_Account{
	use WPJAM_Instance_Trait;

	private $appid;
	private $openid;

	private function __construct($appid, $openid){
		$this->appid	= $appid;
		$this->openid	= $openid;
	}

	public function __get($key){
		if(in_array($key, ['appid', 'openid'])){
			return $this->$key;
		}elseif($key == 'data'){
			return self::get($this->appid, $this->openid);
		}elseif($key == 'time'){
			return $this->data['time'] ?: time();
		}else{
			return $this->data[$key] ?? null;
		}
	}

	public function __isset($key){
		return $this->$key !== null;
	}

	public function get_subscribe($template_id){
		$subscribes	= self::cache_get($this->appid, 'subscribes:'.$this->openid);

		if($subscribes === false){
			$subscribes	= WEAPP_Subscribe_Message::Query()
						->where('appid', $this->appid)
						->where('openid', $this->openid)
						->get_results();

			self::cache_set($this->appid, 'subscribes:'.$this->openid, $subscribes, DAY_IN_SECONDS);
		}

		if($subscribes){
			foreach($subscribes as $subscribe){
				if($subscribe['template_id'] == $template_id){
					return $subscribe;
				}
			}
		}

		return false;
	}

	public function update_subscribe($data){
		self::cache_delete($this->appid, 'subscribes:'.$this->openid);

		if(isset($data['id'])){
			return WEAPP_Subscribe_Message::update($data['id'], ['times'=>$data['times']]);
		}else{
			return WEAPP_Subscribe_Message::insert(array_merge($data, ['appid'=>$this->appid, 'openid'=>$this->openid]));
		}
	}

	public function subscribe($template_id, $times=1){
		$data	= $this->get_subscribe($template_id);

		if($data === false){
			return $this->update_subscribe(['template_id'=>$template_id, 'times'=>$times]); 
		}else{
			if($data['times'] == -1){
				return true;
			}

			$data['times']	= ($times == -1) ? -1 : ($data['times'] + $times);

			return $this->update_subscribe($data);
		}
	}

	public function consume($template_id){
		if($data = $this->get_subscribe($template_id)){
			if($data['times']){
				if($data['times'] != -1){
					$data['times']--;

					return $this->update_subscribe($data);
				}

				return true;
			}
		}

		return false;
	}

	public function save($data){
		if($data){
			if(isset($data['openid']) && $data['openid'] != $this->openid){
				return new WP_Error('openid_modified_not_allowed', '不能修改 openid');
			}

			if($data = wpjam_array_except($data, ['appid', 'blog_id'])){
				return self::update($this->appid, $this->openid, array_merge($data, ['modified'=>time()]));
			}
		}

		return true;
	}

	public function parse_for_json(){
		if(!$this->data){
			return [];
		}

		$json				= [];
		$json['openid']		= $this->openid;
		$json['nickname']	= $this->nickname;
		$json['avatarurl']	= wpjam_get_thumbnail($this->avatarurl, '100x100');
		$json['phone']		= $this->phone ?? '';
		$json['user_id']	= (int)$this->user_id;
		$json['appid']		= $this->appid;
		$json['blog_id']	= (int)$this->blog_id;

		if(isset($this->account_id)){
			$json['account_id']	= (int)$this->account_id;
		}

		if(!isset($this->nickname)){
			trigger_error(var_export($this->data, true));
		}

		if(empty($this->nickname) || empty($this->avatarurl) || (time() - $this->modified > HOUR_IN_SECONDS)){
			$json['expired']	= true;
		}else{
			$json['expired']	= false;
		}

		$json['time']		= (int)$this->time;
		$json['modified']	= (int)$this->modified;

		if(doing_filter('weapp_user_json')){
			return $json;
		}else{
			return apply_filters('weapp_user_json', $json, $this->openid);
		}
	}

	public function generate_access_token(){
		$auth_token	= weapp_get_setting('auth_token', $this->appid);

		if(!$auth_token){
			$auth_token	= wp_generate_password(8, false, false);

			weapp_update_setting('auth_token', $auth_token, $this->appid);
		}

		return wpjam_generate_jwt([
			'appid'		=> $this->appid,
			'openid'	=> $this->openid,
			'exp'		=> time()+WEEK_IN_SECONDS,
			'token'		=> $auth_token,
		]);
	}

	public static function __callStatic($method, $args){
		$appid		= wpjam_array_pull($args, 0);
		$handler	= self::get_handler($appid);

		if($method == 'get'){
			if($data = $handler->get($args[1])){
				$data['appid']		= $appid;
				$data['blog_id']	= get_current_blog_id();
			}

			return $data;
		}elseif($method == 'create'){
			$data	= $args[1];

			if(empty($data['openid'])){
				return WP_Error('empty_openid', 'Openid 不能为空');
			}

			$result	= $handler->insert(array_merge($data, ['time'=>time(), 'modified'=>time()]));

			if(is_wp_error($result)){
				return $result;
			}

			return self::get_instance($appid, $data['openid']);
		}elseif($method == 'sync'){
			$data	= $args[1];
			$data	= [
				'openid'	=> $data['openId'],
				'nickname'	=> $data['nickName'],
				'avatarurl'	=> $data['avatarUrl']
			];

			if(!empty($data['unionId'])){
				$data['unionid']	= $data['unionId'];
			}

			foreach(['nickname'=>255, ] as $key => $length){
				if(isset($data[$key]) && mb_strlen($data['nickname'], 'UTF-8') > $length){
					$data[$key]	= mb_substr($data[$key], 0, $length, 'UTF-8');
				}
			}

			$openid	= $data['openid'];

			if($handler->get($openid)){
				$result	= $handler->update($openid, array_merge($data, ['modified'=>time()]));
			}else{
				$result	= $handler->insert(array_merge($data, ['time'=>time(), 'modified'=>time()]));
			}

			if(is_wp_error($result)){
				return $result;
			}

			return self::get_instance($appid, $openid);
		}elseif($method == 'render'){
			$user	= $args[1];

			if(!is_array($user)){
				$user = self::get($user);
			}

			if($user){
				$user['username'] = $user['nickname'];

				if($user['avatarurl']) {
					$user['username'] = '<img src="'.str_replace('/0', '/64', $user['avatarurl']).'" width="32" />'.$user['username'];
				}
			}

			return $user;
		}elseif(in_array($method, ['get_by_access_token', 'get_openid_by_access_token'])){
			$data	= wpjam_verify_jwt($args[1]);

			if($data === false 
				|| empty($data['openid']) 
				|| empty($data['token']) 
				|| $data['token'] != weapp_get_setting('auth_token', $appid)
			){
				return new WP_Error('illegal_access_token', 'Token 非法或已过期！');
			}

			if(empty($data['appid']) || ($data['appid'] != $appid)){
				return new WP_Error('illegal_appid', 'appid 不匹配！');
			}

			if($method == 'get_by_access_token'){
				return self::get_instance($appid, $data['openid']);
			}else{
				return $data['openid'];
			}
		}elseif($method == 'reset_access_token'){
			return weapp_delete_setting('auth_token', $appid);
		}elseif(strtolower($method) == 'query'){
			if($args){
				return $handler->query($args[0], 'object');
			}else{
				return $handler;
			}
		}else{
			return $handler->$method(...$args);
		}
	}

	public static function get_openid_by($appid, $key, $value){
		return self::Query($appid)->where($key, $value)->get_var('openid');
	}

	public static function get_email($appid, $openid){
		return $openid.'@'.$appid.'.weapp';
	}

	public static function get_instance($appid, $openid){
		return self::instance($appid, $openid);
	}

	public static function get_table($appid){
		return $GLOBALS['wpdb']->base_prefix.'weapp_'.$appid.'_users';
	}

	public static function get_handler($appid){
		return wpjam_get_handler([
			'table_name'		=> self::get_table($appid),
			'pending_queue'		=> 'weapp_account:'.$appid,
			'primary_key'		=> 'openid',
			'cache_group'		=> ['weapp_users', true],
			'cache_prefix'		=> $appid,
			'searchable_fields'	=> ['nickname','openid']
		]);
	}

	public static function create_table($appid){
		require_once(ABSPATH.'wp-admin/includes/upgrade.php');

		$table	= self::get_table($appid);
		$wpdb	= $GLOBALS['wpdb'];

		if($wpdb->get_var("show tables like '{$table}'") != $table) {

			$sql = "
			CREATE TABLE IF NOT EXISTS `{$table}` (
				`openid` varchar(30) NOT NULL,
				`unionid` varchar(30) NOT NULL,
				`nickname` varchar(255)	NOT NULL,
				`avatarurl` varchar(511) NOT NULL,
				`user_id` bigint(20) NOT NULL,
				`phone` bigint(15) NOT NULL,
				`country_code` int(4) NOT NULL DEFAULT 86,
				`time` int(10) NOT NULL,
				`modified` int(10) NOT NULL,
				PRIMARY KEY	(`openid`),
				KEY `user_idx` (`user_id`),
				KEY  `phone_idx` (`phone`, `country_code`)
			) DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;
			";
	 
			dbDelta($sql);
		}

		if($wpdb->get_var("SHOW COLUMNS FROM `{$table}` LIKE 'user_id'") != 'user_id'){
			$wpdb->query("ALTER TABLE {$table} ADD `user_id` BIGINT(20) NOT NULL");
			$wpdb->query("ALTER TABLE `{$table}` ADD KEY `user_idx` (`user_id`);");
		}

		if(!$wpdb->query("SHOW INDEX FROM `{$table}` WHERE Key_name='phone_idx'")){
			$wpdb->query("ALTER TABLE `{$table}` ADD COLUMN `country_code` int(4) NOT NULL DEFAULT 86");
			$wpdb->query("ALTER TABLE `{$table}` CHANGE COLUMN `phone` `phone` bigint(15) NOT NULL");
			$wpdb->query("ALTER TABLE `{$table}` DROP COLUMN `language`");
			$wpdb->query("ALTER TABLE `{$table}` ADD KEY `phone_idx` (`phone`, `country_code`)");
		}

		do_action('weapp_user_table_created', $table);
	}
}

class WEAPP_Bind extends WPJAM_Qrcode_Bind{
	public function __construct($appid){
		parent::__construct('weapp', $appid);

		add_action('wpjam_api',	[$this, 'register_api']);
	}

	public function __call($method, $args){
		return WEAPP_Account::$method($this->appid, ...$args);
	}

	public function get_by_openid($meta_type, $openid){
		$object	= parent::get_by_openid($meta_type, $openid);

		if(!$object){
			if(!$this->weixin_appid){
				return null;
			}

			$unionid	= $this->get_unionid($openid);

			if(empty($unionid)){
				return new WP_Error('empty_unionid','请先授权！');
			}

			$weixin_bind	= wpjam_get_bind_object('weixin', $this->weixin_appid);

			if(!$weixin_bind){
				return new WP_Error('weixin_bind_disabled', '请先开启微信公众号登录');
			}

			$weixin_openid	= call_user_func([$weixin_bind, 'get_openid_by'], 'unionid', $unionid);

			if(empty($weixin_openid)) {
				return new WP_Error('user_not_exists','用户不存在');
			}

			return call_user_func([$weixin_bind, 'get_by_openid'], $weixin_openid, $meta_type);
		}

		return $object;
	}

	public function create_qrcode($key, $args=[]){
		$qrcode	= $this->cache_get($key.'_qrcode');

		if($qrcode === false){
			// $action	= isset($args['id']) ? 'bind' : 'signup';
			$env	= wpjam_get_parameter('env') ?: 'release';
			$action	= $args['action'] ?? 'signup';
			$scene	= wp_generate_password(11, false, false).microtime(true)*10000;
			$args	= ['page'=>$this->get_bind_page(), 'scene'=>$action.'='.$scene, 'type'=>'unlimit', 'width'=>430];
			$args	+= $env != 'release' ? ['env_version'=>$env,	'check_path'=>false] : [];
			$qrcode	= weapp_get_wxacode($args, $this->appid);
			
			if(is_wp_error($qrcode)){
				return $qrcode;
			}

			$qrcode	= [
				'type'		=> 'weapp',
				'key'		=> $key, 
				'code'		=> rand(100000, 999999), 
				'scene'		=> $scene, 
				'qrcode'	=> $qrcode
			];

			if($args){
				$qrcode	= array_merge($args, $qrcode);
			}

			$this->cache_set($key.'_qrcode', $qrcode, 1200);
			$this->cache_set($scene.'_scene', $qrcode, 1200);
		}

		return $qrcode;	
	}

	public function get_user($openid){
		$object	= WEAPP_Account::get_instance($this->appid, $openid);

		return $object ? $object->data : null;
	}

	public function update_user($openid, $data){
		return WEAPP_Account::update($this->appid, $openid, $data);
	}

	public function get_openid_by($key, $value){
		return WEAPP_Account::get_openid_by($this->appid, $key, $value); 
	}

	public function register_api($json){
		$callbacks	= [
			'weapp.qrcode.bind'	=> 'api_qrcode_bind',
			'weapp.qrcode.code'	=> 'api_qrcode_code',
		];

		if(isset($callbacks[$json])){
			wpjam_register_api($json, ['callback' => [$this, $callbacks[$json]]]);
		}
	}

	public function api_qrcode_bind(){
		$scene	= wpjam_get_parameter('scene',	['method'=>'REQUEST',	'required'=>true]);
		$openid	= weapp_get_current_openid();
				
		if(is_wp_error($openid)){
			return $openid;
		}

		$user	= $this->scan_qrcode($openid, $scene);

		if(is_wp_error($user)){
			return $user;
		}

		if(is_numeric($user)){
			return new WP_Error('invalid_scene', '非法场景值');
		}

		return true;
	}

	public function api_qrcode_code(){
		$scene	= wpjam_get_parameter('scene',	['method'=>'REQUEST',	'required'=>true]);
		$openid	= weapp_get_current_openid();
				
		if(is_wp_error($openid)){
			return $openid;
		}

		$code	= $this->scan_qrcode($openid, $scene);

		if(is_wp_error($code)){
			return $code;
		}

		if(!is_numeric($code)){
			return new WP_Error('invalid_scene', '非法场景值');
		}

		return ['code'=>(int)$code];
	}

	public static function get_bind_page(){
		return apply_filters('weapp_bind_page', '');
	}
}

class WEAPP_User{
	use WEAPP_Trait;

	public static function __callStatic($method, $args){
		$appid	= self::get_appid();

		if($method == 'sync_user'){
			$method	= 'sync';
		}

		if(method_exists('WEAPP_Account', $method)){
			$reflection	= new ReflectionClass('WEAPP_Account');
			$static		= $reflection->getMethod($method)->isStatic();
		}else{
			$static	= true;
		}

		if($static){
			$callback	= ['WEAPP_Account', $method];
			$args		= array_merge([$appid], $args);
		}else{
			$openid		= array_shift($args);
			$openid		= is_array($openid) ? $openid['openid'] : $openid;
			$object		= WEAPP_Account::get_instance($appid, $openid);
			$callback	= [$object, $method];
		}

		return call_user_func_array($callback, $args);
	}

	public static function get_views(){
		$views = [];

		foreach([0=>'所有用户', 1=>'已授权用户', -1=>'未授权用户'] as $k => $v){
			$views[$k]	= ['filter'=>['auth'=>$k], 'label'=>$v];
		}

		return $views;
	}

	public static function query_items($limit, $offset){
		$appid	= weapp_get_appid();
		$auth	= wpjam_get_data_parameter('auth');

		if($auth == 1){
			WEAPP_Account::get_handler($appid)->where_not('avatarurl', '');
		}elseif($auth == -1){
			WEAPP_Account::get_handler($appid)->where('avatarurl', '');
		}

		return WEAPP_Account::query_items($appid, $limit, $offset);
	}

	public static function render_item($item){
		$item = self::render($item);

		$item['time']		= get_date_from_gmt(date('Y-m-d H:i:s',$item['time']));
		$item['modified']	= $item['modified']?get_date_from_gmt(date('Y-m-d H:i:s',$item['modified'])):'';

		$item['openid']		= 'OpenID：'.$item['openid'];

		if($item['unionid']){
			$item['openid']	= $item['openid'].'<br />UnionID：'.$item['unionid'];
		}

		if($item['user_id']){
			$item['openid']	= $item['openid'].'<br />USER_ID：'.$item['user_id'];
		}

		return $item;
	}

	public static function get_actions(){
		// return ['reset_access_token'=> ['title'=>'重置Token', 'direct'=>true]];
	}

	public static function get_fields($action_key='', $id=''){
		return [
			'username'	=> ['title'=>'用户',		'type'=>'view',	'show_admin_column'=>true,	'column_style'=>'width:200px;'],
			'openid'	=> ['title'=>'OpenID',	'type'=>'view',	'show_admin_column'=>true],
			'time'		=> ['title'=>'注册时间',	'type'=>'view',	'show_admin_column'=>'only',	'sortable_column'=>true,	'column_style'=>'width:120px;'],
			'modified'	=> ['title'=>'最近访问',	'type'=>'view',	'show_admin_column'=>'only',	'sortable_column'=>true,	'column_style'=>'width:120px;'],
		];
	}
}

trait WEAPP_Trait{
	protected static $appid;
	public static function get_appid($appid=''){
		if(!empty(static::$appid)){
			$appid	=  static::$appid;
		}else{
			$appid	= $appid ?: weapp_get_appid();
		}

		if(is_admin()){
			if(empty($appid)){
				wp_die('小程序 appid 为空',	'empty_appid');
			}else{
				return $appid;
			}
		}else{
			$weapp	= weapp($appid);
			if(is_wp_error($weapp)){
				if(wpjam_is_json_request()){
					trigger_error($weapp->get_error_code().'：'.$weapp->get_error_message());
					wpjam_send_json($weapp);
				}else{
					wp_die($weapp->get_error_message(), $weapp->get_error_code());
				}
			}else{
				return $appid;
			}
		}	
	}

	public static function set_appid($appid=''){
		static::$appid	= $appid ?: weapp_get_appid();
	}

	public static function get_blog_id(){
		if(is_multisite()){
			$appid	= self::get_appid();
			$weapp_setting	= WEAPP_Setting::get($appid);

			if($weapp_setting && isset($weapp_setting['blog_id'])){
				return $weapp_setting['blog_id'];
			}else{
				return 0;
			}
		}else{
			return get_current_blog_id();
		}
	}
}