<?php
class WEAPP_GroupShare extends WPJAM_Model {
	use WEAPP_Trait;
	public static function get_shares($gid){
		return self::get_by('gid', $gid);
	}

	public static function insert($data){
		if(!empty($data['gid'])){
			parent::insert($data);
		}
	}

	public static function update($id, $data){
		if(!empty($data['gid'])){
			parent::update($id, $data);
		}
	}

	protected static $handlers;
	protected static $appid;

	public static function get_table(){
		global $wpdb;
		return $wpdb->base_prefix.'weapp_'.static::get_appid().'_group_shares';
	}

	public static function get_handler(){
		static::$handlers	= (static::$handlers)??array();
		$appid				= static::get_appid();

		if(empty(static::$handlers[$appid])){
			static::$handlers[$appid] = new WPJAM_DB(self::get_table(), array(
				'primary_key'		=> 'id',
				'group_cache_key'	=> 'gid',
				'cache_prefix'		=> $appid,
				'cache_group'		=> ['weapp_group_shares', true],
				'order_by'			=> 'time',
				'field_types'		=> ['time'=>'%d'],
				'searchable_fields'	=> ['text'],
				'filterable_fields'	=> ['gid','openid','type'],
			));
		}

		return static::$handlers[$appid];
	}

	public static function create_table(){
		global $wpdb;

		require_once(ABSPATH.'wp-admin/includes/upgrade.php');

		$table = self::get_table();

		if($wpdb->get_var("show tables like '{$table}'") != $table) {
			$sql = "
			CREATE TABLE IF NOT EXISTS `{$table}` (
				`id` 		bigint(20) NOT NULL auto_increment,
				`openid`	varchar(30) NOT NULL,
				`gid` 		varchar(30) NOT NULL,
				`type`		varchar(15)	NOT NULL,
				`time`		int(10) NOT NULL,
				PRIMARY KEY	(`id`),
				KEY `openid` (`openid`),
				KEY `gid` (`gid`),
				KEY `type` (`type`)
			) DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;
			";
	 
			dbDelta($sql);
		}
	}
}

// 根据参数获取当前群分享和朋友圈数据
function weapp_get_group_shares($args=[]){
	static $groups;

	if(isset($groups))	return $groups;

	$groups	= [];

	$iv = wpjam_get_parameter('iv', array('method' =>'POST'));
	if($iv){	// 用户信息
		$send	= 0;

		include WP_PLUGIN_DIR.'/weapp/api/function.php';
		$user_json	= WEAPP_API::api_auth_signon();
		$user		= $user_json['user'];

		$groups['access_token']	= WEAPP_User::generate_access_token($user['openid']);
		$groups['expired_in']	= DAY_IN_SECONDS - 600;
		$groups['user']			= WEAPP_User::parse_for_json($user);
	}

	$share_type	= wpjam_get_parameter('share_type',	['method'=>'POST']);
	if(empty($share_type))	return [];

	$code		= wpjam_get_parameter('code',	['method'=>'POST']);

	if($code){
		$session_data 	= weapp()->jscode2session($code);
		
		if(is_wp_error($session_data)){
			return $session_data;
		}

		$openid	= $session_data['openid'];
	}else{
		$openid	= weapp_get_current_openid();
	}
	
	if(is_wp_error($openid)){
		return $openid;
	}

	$gid	= wpjam_get_parameter('gid',	array('method'=>'POST'));
	if(!$gid){
		$share_iv 	= wpjam_get_parameter('share_iv',array('method'=>'POST'));

		if($share_iv){
			$session_key	= weapp_get_session_key($code);
			if(is_wp_error($session_key)){
				return $session_key;
			}

			$share_encrypted_data	= wpjam_get_parameter('share_encrypted_data',	array('method'=>'POST', 'required'=>true));

			$gid	= weapp()->decrypt($session_key, $share_iv, $share_encrypted_data);

			if(is_wp_error($gid)){
				return $gid;
			}

			$gid	= $gid['openGId'];
		}
	}

	$type = $share_type;
	if($type == 'timeline'){
		$groups['share_user'] = WEAPP_User::parse_for_json($gid);
	}elseif($type == 'group'){
		$groups['gid']	=  $gid;
	}

	$time	= time();

	$group_shares	= WEAPP_GroupShare::get_shares($gid);

	$data	= compact('openid','gid','type','time');
	if($args){
		$data	= $data+$args; 
	}

	if($group_shares){
		$group_openids	= array_column($group_shares, 'openid');
		$group_shares	= array_combine($group_openids, $group_shares);

		if(isset($group_shares[$openid])){
			$current_share	= $group_shares[$openid];

			// if($time - $current_share['time'] > DAY_IN_SECONDS){
				WEAPP_GroupShare::update($current_share['id'], $data);
			// }
		}else{
			WEAPP_GroupShare::insert($data);
		}
	}else{
		WEAPP_GroupShare::insert($data);
	}

	$groups['group_shares']	= WEAPP_GroupShare::get_shares($gid);

	return $groups;
}