<?php
class WEAPP_Extend extends WPJAM_Register{
	public static function debug_callback($args){
		return strpos($args['caller'], 'weapp') !== false || strpos($args['file'], 'weapp') !== false;
	}

	public static function activation(){
		self::call_active('create_table');
		
		WEAPP_Setting::create_table();
	}

	public static function on_admin_init(){
		if(is_multisite()){
			$settings	= weapp_get_settings();
		}else{
			$setting	= weapp_get_setting();
			$settings	= $setting ? [$setting] : [];
		}

		$count	= count($settings);
		
		$position	= 3.1;

		if($settings){
			foreach($settings as $setting){
				$subs		= [];
				$appid	= $setting['appid'] ?? '';

				if(is_multisite()){
					$base_menu		= $count == 1 ? 'weapp':'weapp-'.$appid;
					$option_name	= 'weapp_'.$appid;
				}else{
					$base_menu		= 'weapp';
					$option_name	= 'wpjam_weapp';
				}

				$subs[$base_menu]	= [
					'menu_title'	=> '小程序设置',	
					'function'		=> 'option',
					'page_file'		=> __DIR__.'/weapp-pages.php',
					'option_name'	=> $option_name,
					'load_callback'	=> 'weapp_load_option_page'
				];

				$subs[$base_menu.'-users']	= [
					'menu_title'	=> '用户管理',	
					'function'		=> 'list',
					'singular'		=> 'weapp-user',
					'plural'		=> 'weapp-users',
					'primary_key'	=> 'openid',
					'model'			=> 'WEAPP_User',
				];

				$subs	= array_merge($subs, self::call_active('get_sub_pages', $base_menu, $appid));

				if(is_multisite() && $position == 3.1){
					$subs['weapp-settings'] = [
						'menu_title'	=> '所有小程序',
						'function'		=> 'list',
						'capability'	=> 'manage_sites',
						'list_table'	=> 'WEAPP_Setting',
					];
				}

				$position	+= 0.01;

				wpjam_add_menu_page($base_menu, [
					'menu_title'	=> $count == 1 ? '微信小程序' : $setting['name'],	
					'icon'			=> 'dashicons-weapp',
					'position'		=> (string)$position,
					'function'		=> 'list',
					'subs'			=> $subs
				]);
			}
		}else{
			if(is_multisite()){
				wpjam_add_menu_page('weapp-settings',[
					'menu_title'	=> '微信小程序',	
					'icon'			=> 'dashicons-weapp',
					'position'		=> (string)$position,
					'capability'	=> 'manage_sites',
					'function'		=> 'list',
					'list_table'	=> 'WEAPP_Setting',
				]);
			}else{
				wpjam_add_menu_page('weapp', [
					'menu_title'	=> '微信小程序',	
					'icon'			=> 'dashicons-weapp',
					'position'		=> (string)$position,
					'function'		=> 'option',
					'page_file'		=> __DIR__.'/weapp-pages.php',
					'option_name'	=> 'wpjam_weapp',
					'load_callback'	=> 'weapp_load_option_page'
				]);
			}
		}

		wp_add_inline_style('list-tables',  "\n".implode("\n", [
			'#adminmenu div.dashicons-weapp:before{font-family: remixicon!important; font-style: normal; -webkit-font-smoothing: antialiased;content: "\ef5a";}',
		])."\n");
	}

	public static function filter_http_response_error_debug($status, $errcode){
		if($status && $errcode == 9410000){	// 直播间为空的错误别记录了
			return false;
		}

		return true;
	}

	protected static function get_config($key){
		if(in_array($key, ['register_json', 'init'])){
			return true;
		}
	}
}

function weapp_register_extend($name, $args){
	return WEAPP_Extend::register($name, $args);
}

function weapp_bind_extend(){
	$appid	= weapp_get_current_blog_appid();

	if($appid){
		wpjam_register_bind('weapp', $appid, 'WEAPP_Bind');

		if(WEAPP_Bind::get_bind_page()){
			wpjam_register_user_signup('weapp', [
				'title'			=> '微信小程序',
				'login_title'	=> '微信小程序扫码登录',	
				'appid'			=> $appid,
				'model'			=> 'WPJAM_User_Qrcode_Signup',
				'bind'			=> true,
				'login'			=> true
			]);
		}
	}
}

weapp_register_extend('hook', [
	'title'	=> 'Hook',
	'model'	=> 'WEAPP_Hook',
	'file'	=> __DIR__.'/weapp-hook.php',
]);

weapp_register_extend('bind', [
	'title'	=> '小程序用户绑定',
	'init'	=> 'weapp_bind_extend',
]);

weapp_register_extend('plugin', [
	'title'	=> '小程序插件',
	'model'	=> 'WEAPP_Plugin',
	'file'	=> __DIR__.'/weapp-plugins.php',
]);

weapp_register_extend('path', [
	'title'	=> '小程序路径和二维码',
	'model'	=> 'WEAPP_Path',
	'file'	=> __DIR__.'/weapp-path.php',
]);

weapp_register_extend('tabbar', [
	'title'	=> '小程序底部菜单栏',
	'model'	=> 'WEAPP_Tabbar_Extend',
	'file'	=> __DIR__.'/weapp-tabbar.php',
]);

weapp_register_extend('live-broadcast', [
	'title'	=> '小程序直播',
	'model'	=> 'WEAPP_Live_Broadcast',
	'file'	=> __DIR__.'/weapp-live-broadcast.php',
]);

weapp_register_extend('template', [
	'title'	=> '小程序订阅消息模板',
	'model'	=> 'WEAPP_Subscribe_Message',
	'file'	=> __DIR__.'/weapp-template.php',
]);

weapp_register_extend('webview-domain', [
	'title'	=> '小程序业务域名',
	'model'	=> 'WEAPP_Webview_Domain',
]);

weapp_register_extend('sec', [
	'title'	=> '小程序安全',
	'model'	=> 'WEAPP_Sec',
]);

weapp_register_extend('minishop', [
	'title'	=> '微信小商店',
	'model'	=> 'WEAPP_Minishop',
]);

if(did_action('wpjam_debug_loaded')){
	wpjam_register_debug_type('weapp', [
		'order'		=> 18,
		'name'		=> '小程序插件警告',
		'callback'	=> ['WEAPP_Extend', 'debug_callback']
	]);
}

if(is_weapp()){
	remove_action('set_comment_cookies', 'wp_set_comment_cookies', 10, 3);
	remove_action('sanitize_comment_cookies', 'sanitize_comment_cookies');

	remove_filter('determine_current_user', 'wp_validate_auth_cookie');
	remove_filter('determine_current_user', 'wp_validate_logged_in_cookie', 20);
}

add_filter('wpjam_http_response_error_debug',	['WEAPP_Extend', 'filter_http_response_error_debug'], 10, 2);

if(is_admin()){
	register_activation_hook(WEAPP_PLUGIN_FILE , ['WEAPP_Extend', 'activation']);

	add_action('wpjam_admin_init',	['WEAPP_Extend', 'on_admin_init']);
}