<?php
class WEAPP_Hook{
	public static function init(){
		add_filter('get_avatar_data',	[self::class, 'filter_avatar_data']);
		add_filter('the_comments',		[self::class, 'filter_comments']);

		if(is_weapp()){
			add_filter('wpjam_current_user',	[self::class, 'filter_current_user']);
		}
	}

	public static function filter_current_user($current_user){
		if(!weapp_get_setting()){
			return $current_user;
		}

		$openid	= weapp_get_current_openid();

		if(is_wp_error($openid)){
			return $openid;
		}

		$appid	= weapp_get_current_appid();

		$account	= weapp_get_user_object($openid, $appid);
		$weapp_user	= $account->parse_for_json();

		$user_id	= $weapp_user['user_id'] ?? 0;
		$wp_user 	= $user_id ? get_userdata($user_id) : null;

		if(!$user_id || !$wp_user){
			$weapp_user['user_id']	= 0;
		}

		$weapp_user['platform']		= 'weapp';
		$weapp_user['user_email']	= $openid.'@'.$appid.'.weapp';

		return $weapp_user;
	}

	public static function filter_avatar_data($args){
		$email	= $args['email'] ?? '';

		if($email && str_ends_with($email, '.weapp')){
			$parts	= explode('@', $email);
			$openid	= $parts[0];
			$appid	= wpjam_remove_postfix($parts[1], '.weapp');
			$user	= weapp_get_user_object($openid, $appid);
			$url	= $user ? $user->avatarurl : '';

			if($url){
				$args['url']	= $url;
			}
		}

		return $args;
	}

	public static function filter_comments($comments){
		$list	= [];

		foreach($comments as $comment){
			$email	= $comment->comment_author_email;

			if(strrpos($email, '.weapp')){
				$parts	= explode('@', $email);
				$openid	= $parts[0];
				$appid	= str_replace('.weapp', '', $parts[1]);

				$list[$appid]	= $list[$appid] ?? [];
				$list[$appid][]	= $openid;
			}
		}

		foreach($list as $appid => $openids){
			wpjam_lazyload('weapp_account:'.$appid, $openids);
		}

		return $comments;
	}

	public static function register_json($json){
		if($json == 'weapp.token'){
			return wpjam_register_json($json, ['grant'=>true, 'callback'=>[self::class, 'api_token']]);
		}

		$callback	= 'api_'.str_replace(['weapp.','.'], ['','_'], $json);

		if(method_exists(self::class, $callback)){
			return wpjam_register_json($json, ['callback'=>[self::class, $callback]]);
		}
	}

	public static function api_user_login(){
		$appid	= weapp_get_appid();
		$weapp	= weapp($appid);
		$code	= wpjam_get_parameter('code',	['method'=>'POST',	'required'=>true]);

		$session_data	= $weapp->jscode2session($code);

		if(is_wp_error($session_data)){
			return $session_data;
		}

		if(is_null($session_data) || empty($session_data['openid'])){
			return new WP_Error('invalid_session_data', '非法 Session Data');
		}

		$openid		= $session_data['openid'];
		$unionid	= $session_data['unionid'] ?? ($session_data['unionId'] ?? '');
		$user		= WEAPP_Account::get($appid, $openid);

		if(!$user || empty($user['openid'])){
			$account	= WEAPP_Account::create($appid, ['openid'=>$openid,	'unionid'=>$unionid]);

			if(is_wp_error($account)){
				return $account;
			}
		}else{
			$account	= weapp_get_user_object($openid);

			if($unionid && $unionid != $user['unionid']){
				$account->save(['unionid'=>$unionid]);
			}
		}

		weapp_set_current_openid($openid);

		$user	= $account->parse_for_json();

		do_action('weapp_user_signuped', $user);

		$current_user	= wpjam_get_current_user();
		$access_token	= $account->generate_access_token();

		return [
			'access_token'	=> $access_token,
			'expired_in'	=> WEEK_IN_SECONDS,
			'user'			=> $current_user,
			'current_user'	=> $current_user,
		];
	}

	public static function api_user_profile(){
		$openid	= weapp_get_current_openid();

		if(is_wp_error($openid)){
			return $openid;
		}

		$user	= wpjam_get_parameter('user',	['method'=>'POST',	'required'=>true]);

		if(is_string($user)){
			$user	= wpjam_json_decode($user);
		}

		if($user['nickName'] != '微信用户'){
			$account	= WEAPP_Account::sync(weapp_get_appid(), array_merge($user, ['openId'=>$openid]));
		}else{
			$account	= weapp_get_user_object($openid);
		}

		if(is_wp_error($account)){
			return $account;
		}

		$user	= $current_user	= apply_filters('wpjam_current_user', $account->parse_for_json());

		return ['user'=>$user,	'current_user'=>$current_user];
	}

	public static function api_user_info(){
		// $new_api	= current_time('mysql') > '2021-04-12 11:59:00';
		
		// if($new_api){	// 4.13 之后微信加密数据解析出来的头型是灰色头型，昵称是「微信用户」
		// 	$openid	= weapp_get_current_openid();

		// 	if(is_wp_error($openid)){
		// 		return $openid;
		// 	}

		// 	$account	= weapp_get_user_object($openid);

		// 	return ['user'=>$account->parse_for_json()];
		// }

		$iv			= wpjam_get_parameter('iv',				['method'=>'POST',	'required'=>true]);
		$encrypted	= wpjam_get_parameter('encrypted_data',	['method'=>'POST',	'required'=>true]);

		$session_key	= weapp_get_session_key();

		if(is_wp_error($session_key)){
			return $session_key;
		}

		$weapp		= weapp();
		$user_info	= $weapp->decrypt($session_key, $iv, $encrypted);

		if(is_wp_error($user_info)){
			return new WP_Error('bad_authentication');
		}

		if($user_info['nickName'] != '微信用户'){
			$account	= WEAPP_Account::sync(weapp_get_appid(), $user_info);

			if(is_wp_error($account)){
				return $account;
			}

			return ['user'=>$account->parse_for_json()];
		}else{
			$openid	= weapp_get_current_openid();
			
			if(is_wp_error($openid)){
				return $openid;
			}

			$account	= weapp_get_user_object($openid);

			return ['user'=>array_merge($account->parse_for_json(), ['expired'=>false])];
		}
	}

	public static function api_user_update(){
		$openid	= weapp_get_current_openid();

		if(is_wp_error($openid)){
			return $openid;
		}

		$account	= weapp_get_user_object($openid);

		$data	= [
			'nickname'	=> wpjam_get_parameter('nickname',	['method'=>'POST']),
			'avatarurl'	=> wpjam_get_parameter('avatarurl',	['method'=>'POST']),
		];

		$data	= array_filter($data, 'is_populated');
		$data	= apply_filters('weapp_user_update_json', $data, $openid);

		if($data){
			$account->save($data);
		}

		return ['user'=>$account->parse_for_json()];
	}

	public static function api_user_phone(){
		$code	= wpjam_get_parameter('code',	['method'=>'POST']);

		if($code){
			$response	= weapp()->post('/wxa/business/getuserphonenumber', compact('code'));

			if(is_wp_error($response)){
				return $response;
			}

			$phone_info	= $response['phone_info'];
		}else{
			$iv			= wpjam_get_parameter('iv',				['method'=>'POST',	'required'=>true]);
			$encrypted	= wpjam_get_parameter('encrypted_data',	['method'=>'POST',	'required'=>true]);

			$session_key	= weapp_get_session_key();

			if(is_wp_error($session_key)){
				return $session_key;
			}

			$phone_info	= weapp()->decrypt($session_key, $iv, $encrypted);

			if(is_wp_error($phone_info)){
				return $phone_info;
			}
		}			

		$data	= [
			'phone'			=> $phone_info['purePhoneNumber'],
			'country_code'	=> $phone_info['countryCode'] ?: 86,
		];

		$openid	= weapp_get_current_openid();

		if(!is_wp_error($openid)){
			$account	= weapp_get_user_object($openid);

			$account->save($data);
		}

		return $data+['phone_full'=>$phone_info['phoneNumber']];
	}

	public static function api_auth_signon(){
		$code			= wpjam_get_parameter('code',	['method'=>'POST',	'required'=>true]);
		$session_key	= weapp_get_session_key($code);

		if(is_wp_error($session_key)){
			wpjam_send_json($session_key);
		}

		$iv			= wpjam_get_parameter('iv',				['method'=>'POST',	'required'=>true]);
		$encrypted	= wpjam_get_parameter('encrypted_data',	['method'=>'POST',	'required'=>true]);
		$user_info	= weapp()->decrypt($session_key, $iv, $encrypted);

		if(is_wp_error($user_info)){
			wpjam_send_json($user_info);
		}

		$appid		= weapp_get_appid();
		$account	= WEAPP_Account::sync($appid, $user_info);

		if(is_wp_error($account)){
			wpjam_send_json($account);
		}

		$user	= $account->parse_for_json();

		return [
			'access_token'	=> $account->generate_access_token(),
			'expired_in'	=> WEEK_IN_SECONDS - 600,
			'user'			=> $user,
		];
	}

	public static function api_token(){
		$appid	= weapp_get_current_blog_appid();
		$weapp	= weapp($appid);

		$response	= $weapp->get_access_token(false, 'array');

		wpjam_send_json([
			'errcode'		=> 0,
			'access_token'	=> $response['access_token'],
			'expired_in'	=> $response['expires_at']-time(),
		]);
	}

	public static function api_access_token(){
		if($_SERVER['RAW_HTTP_HOST'] != gethostbyname(gethostname()) ){
			wpjam_send_json(['errcode'=>'invalid', 'errmsg'=>'非法请求']);	// 请使用内网IP访问，HTTP_HOST 可以伪造，存在风险。
		}

		$appid	= weapp_get_appid();
		$weapp	= weapp();

		$access_token	= $weapp->get_access_token();

		wpjam_send_json([
			'errcode'		=>0,
			'access_token'	=> $access_token
		]);
	}
}