<?php
class WEAPP_Live_Broadcast{
	public static function register_json($json){
		if(in_array($json, ['live.rooms', 'weapp.live.rooms'])){
			return wpjam_register_api($json, ['callback'=>[self::class, 'api_live_rooms']]);
		}
	}

	public static function api_live_rooms(){
		$response	= weapp_get_live_rooms();

		if(is_wp_error($response)){
			return $response;
		}

		$live_rooms	= [];
		$room_info	= $response ? $response['room_info'] : [];
		$status		= wpjam_get_parameter('status');

		foreach($room_info as $live_room){
			if(($status && $live_room['live_status'] == $status) || (!$status && in_array($live_room['live_status'], [101,102,103]))){
				$live_room['start_time']	= $live_room['start_time'] ? wpjam_date($live_room['start_time']) : '';
				$live_room['end_time']		= $live_room['end_time'] ? wpjam_date($live_room['end_time']) : '';

				$live_rooms[]	= $live_room;
			}
		}

		if(!$status){
			$live_rooms	= wp_list_sort($live_rooms, 'live_status');
		}

		$banner	= weapp_get_setting('live_rooms_banner');
		$banner	= $banner ? wpjam_get_thumbnail($banner, [1200]) : wpjam_get_thumbnail(WEAPP_PLUGIN_URL.'static/live-rooms.png');

		return [
			'live_rooms'	=> $live_rooms,
			'banner'		=> $banner
		];
	}

	public static function get_sub_pages($base_menu, $appid){
		if(weapp_has_funcscope(52)){
			return [$base_menu.'-live-broadcast'	=> [
				'menu_title'	=> '小程序直播',	
				'function'		=> 'tab',
				'load_callback'	=> [self::class, 'page_load']
			]];
		}
	}

	public static function page_load(){
		wpjam_register_plugin_page_tab('live-rooms',	[
			'tab_title'			=> '直播间',
			'function'			=> 'list',
			'title'				=> '小程序直播间',
			'singular'			=> 'weapp-live-room',
			'plural'			=> 'weapp-live-rooms',
			'model'				=> 'WEAPP_Live_Room',
			'primary_column'	=> 'room_info',
			'per_page'			=> 10
		]);

		wpjam_register_plugin_page_tab('goods',		[
			'title'				=> '商品库',
			'function'			=> 'list',
			'title'				=> '直播间商品库',
			'singular'			=> 'weapp-broadcast-good',
			'plural'			=> 'weapp-broadcast-goods',
			'model'				=> 'WEAPP_Broadcast_Good',
			'primary_column'	=> 'name',
			'query_args'		=> ['status'],
			'per_page'			=> 100
		]);
	}

	public static function path_callback($args, $name){
		if($name == 'live_room'){
			if($room_id = (int)$args['room_id']){
				return 'plugin-private://wx2b03c6e691cd7370/pages/live-player-plugin?room_id='.$room_id;
			}

			return new WP_Error('invalid_room_id', '无效的房间号');
		}elseif($name == 'channel'){
			return [
				'type'				=> 'channel',
				'channel_type'		=> $args['channel_type'],
				'finder_username'	=> $args['finder_username'],
				'feed_id'			=> $args['feed_id'] ?? ''
			];
		}
	}

	public static function init(){
		$appid	= weapp_get_current_blog_appid();

		if(!$appid){
			return;
		}

		if(!is_admin() || (is_admin() && weapp_has_plugin('wx2b03c6e691cd7370', $force=false, $appid))){
			weapp_register_path('live_room',	[
				'title'		=> '进入直播间',
				'group'		=> ['key'=>'external',	'title'=>'跳转外部链接'],
				'callback'	=> [self::class, 'path_callback'],
				'fields'	=> ['room_id'=> ['type'=>'number',	'class'=>'all-options',	'required',	'placeholder'=>'请输入直播间房间号']]
			]);
		}

		weapp_register_path('channel',	[
			'title'		=> '跳转视频号',
			'path'		=> false,
			'group'		=> 'external',
			'callback'	=> [self::class, 'path_callback'],
			'fields'	=> [
				'channel_type'		=> ['type'=>'select',	'options'=>['activity'=>'视频','live'=>'直播间','live_notice'=>'直播预告']],
				'finder_username'	=> ['type'=>'text',		'class'=>'all-options',	'required',	'placeholder'=>'请输入要跳转的视频号ID'],
				'feed_id'			=> ['type'=>'text',		'show_if'=>['key'=>'channel_type',	'value'=>'activity'],	'placeholder'=>'请输入要跳转视频号的视频Feed ID。'],
				'channel_view'		=> ['type'=>'view',		'value'=>'视频号 ID 及视频 Feed ID 请登录 <a target="_blank" href="https://channels.weixin.qq.com">https://channels.weixin.qq.com</a> 获取']
			]
		]);
	}
}

class WEAPP_Live_Room{
	public static function get($room_id){
		return ['roomId'=>$room_id];
	}

	public static function insert($data){
		foreach (['coverImg'=>['label'=>'背景图','size'=>'1080x1920'], 'shareImg'=>['label'=>'分享图','size'=>'800x640']] as $key=>$value) {
			if(empty($data[$key])){
				return new WP_Error('empty_'.$key, $value['label'].'不能为空');
			}

			add_filter('wpjam_thumbnail_args', function($args){
				return $args+['webp'=>0];
			});

			$file	= wpjam_download_image(wpjam_get_thumbnail($data[$key], $value['size']));

			if(is_wp_error($file)){
				return $file;
			}

			$meida = weapp_upload_media($file);

			if(is_wp_error($meida)){
				return $meida;
			}

			unlink($file);

			$data[$key]	= $meida['media_id'];
		}

		$data['name']			= mb_strimwidth($data['name'], 0, 17);
		$data['type']			= intval($data['type']);
		$data['screenType']		= intval($data['screenType']);
		$data['startTime']		= strtotime(get_gmt_from_date($data['startTime']));
		$data['endTime']		= strtotime(get_gmt_from_date($data['endTime']));
		$data['anchorName']		= mb_strimwidth($data['anchorName'], 0, 15);
		$data['anchorWechat']	= $data['anchorWechat'];
		$data['closeLike']		= intval($data['closeLike']);
		$data['closeGoods']		= intval($data['closeGoods']);
		$data['closeComment']	= intval($data['closeComment']);

		$result	= weapp()->post('/wxaapi/broadcast/room/create', $data);

		if(is_wp_error($result)){
			return $result;
		}

		return $result['roomId'];
	}

	public static function add_goods($room_id, $data){
		$ids	= $data['ids'];

		if(empty($ids)){
			return new WP_Error('empty_ids', '直播商品不能为空');
		}

		$result	= weapp()->post('/wxaapi/broadcast/room/addgoods', ['ids'=>$ids, 'roomId'=>$room_id]);;

		if(is_wp_error($result)){
			return $result;
		}else{
			return true;
		}
	}

	public static function query_items($count, $offset){
		if(!weapp_has_plugin('wx2b03c6e691cd7370') && !weapp_has_plugin('wx2b03c6e691cd7370', $force=true)){
			return new WP_Error('weapp_live_plugin_required', '小程序还没有直播权限，请先到小程序后台申请。');
		}else{
			$response	= weapp_get_live_rooms(['start'=>$offset, 'limit'=>$count], true);

			if(is_wp_error($response)){
				if($response->get_error_code() == '48001'){
					return new WP_Error('weapp_live_plugin_required', '小程序还没有直播权限，请先到小程序后台申请。');
				}else{
					return $response;
				}
			}

			$items	= $response['room_info'];
			$total	= $response['total'];		

			return compact('items', 'total');
		}	
	}

	public static function render_item($item){
		$item['share_img']	= $item['share_img'] ? '<img referrerpolicy="no-referrer" src="'.$item['share_img'].'" width="200" height="150" />' : '';

		$room_info	= ['房间号：<strong>'.$item['roomid'].'</strong>'];

		if(!empty($item['start_time'])){
			$room_info[]	= '开&emsp;播：'.get_date_from_gmt(date('Y-m-d H:i:s', $item['start_time']));
		}

		if(!empty($item['end_time'])){
			$room_info[]	= '结&emsp;束：'.get_date_from_gmt(date('Y-m-d H:i:s', $item['end_time']));
		}

		if(!empty($item['anchor_name'])){
			$room_info[]	= '主&emsp;播：'.$item['anchor_name'];
		}

		$item['room_info']	= implode('<br />', $room_info);

		if(!in_array($item['live_status'], [101,102])){
			unset($item['row_actions']['add_goods']);
		}

		return $item;
	}

	public static function get_primary_key(){
		return 'roomid';
	}

	public static function get_actions(){
		return [
			// 'add'		=> ['title'=>'创建',		'page_title'=>'创建直播间',	'response'=>'list'],
			'add_goods'	=> ['title'=>'导入商品',	'page_title'=>'导入商品',		'response'=>'list'],
		];
	}

	public static function get_fields($action_key='', $tid=''){
		if($action_key == ''){
			$live_statuses = [
				101	=> '直播中',
				102 => '未开始',
				103	=> '已结束',
				104 => '禁播',
				105 => '暂停中',
				106	=> '异常',
				107	=> '已过期'
			];

			return [
				'share_img'		=> ['title'=>'分享图',	'type'=>'view',	'show_admin_column'=>true],
				'room_info'		=> ['title'=>'直播间',	'type'=>'view',	'show_admin_column'=>true],
				'live_status'	=> ['title'=>'状态',		'type'=>'view',	'show_admin_column'=>true,	'options'=>$live_statuses],
			];
		}elseif($action_key == 'add'){
			return [
				'name'		=> ['title'=>'名称',		'type'=>'text',		'required',	'placeholder'=>'最短3个汉字，最长17个汉字'],
				'coverImg'	=> ['title'=>'背景图',	'type'=>'img',		'item_type'=>'url',	'description'=>'建议像素1080*1920，大小不超过2M'],
				'shareImg'	=> ['title'=>'分享图',	'type'=>'img',		'item_type'=>'url',	'description'=>'建议像素800*640，大小不超过1M'],
				'type'		=> ['title'=>'类型',		'type'=>'radio',	'options'=>[1=>'推流', 0=>'手机直播']],
				'screenType'=> ['title'=>'横竖屏',	'type'=>'radio',	'options'=>[1=>'横屏', 0=>'竖屏']],
				'time_set'	=> ['title'=>'计划时间',	'type'=>'fieldset',	'fields'=>[
					'startTime'	=> ['title'=>'开播时间',	'type'=>'datetime-local',	'class'=>'',	'required',	'description'=>''],
					'endTime'	=> ['title'=>'结束时间',	'type'=>'datetime-local',	'class'=>'',	'required',	'description'=>'<br/>开播时间需要在当前时间的10分钟后 并且开始时间不能在6个月后<br/>开播时间和结束时间间隔不得短于30分钟，不得超过24小时'],
				]],
				'anchor_set'=> ['title'=>'主播',		'type'=>'fieldset',	'fields'=>[
					'anchorName'	=> ['title'=>'昵称',		'type'=>'text',	'class'=>'all-options',	'required',	'description'=>'2-15个汉字'],
					'anchorWechat'	=> ['title'=>'微信号',	'type'=>'text',	'class'=>'all-options',	'required',	'description'=>'<br />如未实名认证，需要先前往“小程序直播”小程序进行实名验证'],
				]],
				'otherset'	=> ['title'=>'其他设置',	'type'=>'fieldset',	'fields'=>[
					'closeLike'		=> ['title'=>'',	'type'=>'checkbox',	'description'=>'关闭点赞，直播开始后不允许开启'],
					'closeGoods'	=> ['title'=>'',	'type'=>'checkbox',	'description'=>'关闭货架，直播开始后不允许开启'],
					'closeComment'	=> ['title'=>'',	'type'=>'checkbox',	'description'=>'关闭评论，直播开始后不允许开启'],
				]]
			];
		}elseif($action_key == 'add_goods'){
			return [
				'ids'	=> ['title'=>'直播商品ID',	'type'=>'mu-text']
			];
		}
	}
}

class WEAPP_Broadcast_Good{
	public static function get($goods_id){
		$result	= weapp()->post('/wxa/business/getgoodswarehouse', ['goods_ids'=>[$goods_id]]);

		if(is_wp_error($result)){
			return $result;
		}

		if($result['goods']){
			$goods = current($result['goods']);
			$goods['goodsId']		= $goods['goods_id'];
			$goods['coverImgUrl']	= $goods['cover_img_url'];
			$goods['priceType']		= $goods['price_type'];
			$goods['thirdPartyTag']	= $goods['third_party_tag'];

			return $goods;
		}else{
			return [];
		}	
	}

	public static function delete($goods_id){
		return weapp()->post('/wxaapi/broadcast/goods/delete', ['goodsId'=>$goods_id]);
	}

	public static function resetaudit($goods_id){
		$audit_goods	= get_option('weapp_broadcast_audit_goods');
		if($audit_goods && isset($audit_goods[$goods_id])){
			$result = weapp()->post('/wxaapi/broadcast/goods/resetaudit', ['goodsId'=>$goods_id, 'auditId'=>$audit_goods[$goods_id]]);

			if(is_wp_error($result)){
				return $result;
			}else{
				return true;
			}
		}else{
			return new WP_Error('invalid_goods_id', '系统未记录审核ID，无法撤回');
		}
	}

	public static function audit($goods_id){
		$result = weapp()->post('/wxaapi/broadcast/goods/audit', ['goodsId'=>$goods_id]);

		return is_wp_error($result) ? $result : true;
	}

	public static function get_views(){
		$views	= [];
	
		foreach(self::get_status_list() as $status => $label){
			$views[$status]	= ['filter'=>['status'=>$status], 'label'=>$label];
		}

		return $views;
	}

	public static function query_items($limit, $offset){
		if(!weapp_has_plugin('wx2b03c6e691cd7370') && !weapp_has_plugin('wx2b03c6e691cd7370', $force=true)){
			return new WP_Error('weapp_live_plugin_required', '小程序还没有直播权限，请先到小程序后台申请。');
		}else{
			$status		= wpjam_get_data_parameter('status') ?? 2;

			if(get_transient('audit_goods_verified') === false){
				set_transient('audit_goods_verified', 1, HOUR_IN_SECONDS);

				if($audit_goods = get_option('weapp_broadcast_audit_goods')){
					$goods_ids	= array_keys($audit_goods);
					$result		= weapp()->post('/wxa/business/getgoodswarehouse', ['goods_ids'=>$goods_ids]);
					if(!is_wp_error($result)){
						$update	= false; 

						foreach ($result['goods'] as $good) {
							if($good['audit_status'] == 2){
								$update	= true; 
								unset($audit_goods[$good['goods_id']]);
							}
						}

						if($update){
							if($audit_goods){
								update_option('weapp_broadcast_audit_goods', $audit_goods);
							}else{
								delete_option('weapp_broadcast_audit_goods');
							}
						}
					}
				}
			}

			$response	= weapp()->get('/wxaapi/broadcast/goods/getapproved', ['status'=>$status, 'offset'=>$offset, 'limit'=>$limit]);

			if(is_wp_error($response)){
				if($response->get_error_code() == '48001'){
					return new WP_Error('weapp_live_plugin_required', '小程序还没有直播权限，请先到小程序后台申请。');
				}else{
					return $response;
				}
			}

			$items	= $response['goods'];
			$total	= $response['total'];		

			return compact('items', 'total');
		}	
	}

	public static function item_callback($item){
		$item['cover']	= '<img referrerpolicy="no-referrer" src="'.$item['coverImgUrl'].'" width="100" height="100">';
		$item['source']	= $item['thirdPartyTag'] == 2 ? '接口' : '直播控制台添加';

		if($item['priceType'] == 1){
			$price	= '价格：'.$item['price'];
		}elseif($item['priceType'] == 2){
			$price	= '价格：'.$item['price'].' ~ '.$item['price2'];
		}elseif($item['priceType'] == 3){
			$price	= '原价：'.$item['price'].'<br />现价：'.$item['price2'];
		}

		$item['name']	= wpautop($item['name']."\n\n".'直播商品 ID：'.$item['goodsId']);

		if(isset($item['audit_status'])){
			$status	= $item['audit_status'];
		}else{
			$status	= wpjam_get_data_parameter('status') ?? 2;
		}

		if($status == 3){
			unset($item['row_actions']['update']);
		}

		if($status == 1){
			unset($item['row_actions']['update']);
			unset($item['row_actions']['delete']);
		}else{
			unset($item['row_actions']['resetaudit']);
		}

		if($status != 0){
			unset($item['row_actions']['audit']);
		}

		$item['price']	= $price;

		return $item;
	}

	public static function get_primary_key(){
		return 'goodsId';
	}

	public static function get_actions(){
		return [
			'resetaudit'	=> ['title'=>'撤回审核',	'page_title'=>'撤回审核',		'direct'=>true,	'confirm'=>true, 'response'=>'list'],
			'audit'			=> ['title'=>'重新审核',	'page_title'=>'重新提交审核',	'direct'=>true,	'confirm'=>true, 'response'=>'list'],
			// 'update'		=> ['title'=>'编辑',		'page_title'=>'编辑商品',		'response'=>'list'],
			'delete'		=> ['title'=>'删除',		'page_title'=>'删除商品',		'direct'=>true,	'confirm'=>true],
		];
	}

	public static function get_status_list(){
		return [
			2	=> '已审核',
			1	=> '审核中',
			0	=> '未审核',
			3	=> '已驳回'
		];
	}

	public static function get_price_types(){
		return [
			1 	=> '一口价',
			2	=> '价格区间',
			3	=> '折扣价'
		];

		// 1:一口价，此时读price字段; 2:价格区间，此时price字段为左边界，price2字段为右边界; 3:折扣价，此时price字段为原价，price2字段为现价；
	}

	public static function get_fields($action_key='', $tid=''){
		if($action_key == ''){
			return [
				'cover'		=> ['title'=>'图片',	'type'=>'view',	'show_admin_column'=>'only'],
				'name'		=> ['title'=>'商品',	'type'=>'view',	'show_admin_column'=>'only'],
				'price'		=> ['title'=>'价格',	'type'=>'text',	'show_admin_column'=>'only'],
				'url'		=> ['title'=>'路径',	'type'=>'text',	'show_admin_column'=>'only'],
				'source'	=> ['title'=>'来源',	'type'=>'view',	'show_admin_column'=>'only'],
			];
		}elseif($action_key == 'add'){
			return [
				'post_id'	=> ['title'=>'商品',	'type'=>'text',	'data_type'=>'post_type',	'post_type'=>'product'],
				'name'		=> ['title'=>'名称',	'type'=>'text',	'placeholder'=>'不填则使用商品标题，最多12个汉字'],
			];
		}
	}
}

function weapp_get_live_rooms($args=[], $force=false, $appid=''){
	$args	= wp_parse_args($args, [
		'start'		=> 0,
		'limit'		=> 100,
	]);

	$appid	= $appid ?: weapp_get_current_appid();
	$weapp	= weapp($appid);

	if($force){
		wp_cache_set('last_changed', microtime(), 'weapp_live_rooms'.$appid);
	}

	$last_changed	= wp_cache_get_last_changed('weapp_live_rooms'.$appid);
	$cahce_key		= 'live_rooms:'.$last_changed.':'.maybe_serialize($args);

	$live_rooms	= $weapp->cache_get($cahce_key);

	if($live_rooms === false){
		$live_rooms	= $weapp->post('/wxa/business/getliveinfo', $args);

		if(is_wp_error($live_rooms)){
			if($live_rooms->get_error_code() == 1){
				$live_rooms	= []; 
			}else{
				return $live_rooms;
			}
		}

		$weapp->cache_set($cahce_key, $live_rooms, HOUR_IN_SECONDS);
	}

	return $live_rooms;
}