<?php
function weapp_load_option_page(){
	$sections		= [];
	$weapp_appid	= weapp_get_current_appid();
	$option_name	= wpjam_get_plugin_page_setting('option_name');

	if(is_multisite()){
		$basic_fields	= [
			'appid'		=> ['title'=>'小程序ID',		'type'=>'view', 'value'=>$weapp_appid],
			'secret'	=> ['title'=>'小程序密钥',	'type'=>'view',	'value'=>weapp_get_setting('secret')],
		];
	}else{
		$basic_fields	= [
			'appid'		=> ['title'=>'小程序ID',		'type'=>'text',	'required'=>'required',	'class'=>'all-options'],
			'secret'	=> ['title'=>'小程序密钥',	'type'=>'text',	'required'=>'required'],
		];
	}

	$basic_fields['grant_type']	= ['title'=>'小程序类型',	'type'=>'select',	'options'=>[''=>'企业主体','single'=>'个人主体']];

	$sections['basic']	= ['title'=>'基本设置', 'fields'=>$basic_fields];

	if($weapp_appid){
		$options = [
			''						=> '关闭',
			'weapp'					=> '小程序客服',
			'work_weixin'			=> '企业微信客服',
			'weixin_custom_service'	=> '微信客服',
			'custom_reply'			=> '自定义回复'
		];

		$feature_fields	= [
			'appid'					=> ['title'=>'小程序ID',		'type'=>'view',		'value'=>$weapp_appid],
			'contact'				=> ['title'=>'客服设置',		'type'=>'select', 	'options'=>$options],
			'work_weixin_plugid'	=> ['title'=>'企业微信客服ID','type'=>'text',		'placeholder'=>'请输入在企业微信管理后台配置的客服ID',	'show_if'=>['key'=>'contact', 'value'=>'work_weixin']],
			'custom_service_set'	=> ['title'=>'微信客服',	'type'=>'fieldset',	'group'=>true,	'show_if'=>['key'=>'contact', 'value'=>'weixin_custom_service'],	'fields'=>[
				'corp_id'	=> ['title'=>'企业ID',	'type'=>'text',	'class'=>''],
				'url'		=> ['title'=>'客服链接',	'type'=>'url'],
			],	'description'=>'获取企业ID和客服链接：企业微信 - 应用 - 微信客服'],
			'custom_reply_set'		=> ['title'=>'自定义回复',	'type'=>'fieldset',	'show_if'=>['key'=>'contact', 'value'=>'custom_reply'],	'fields'=>[
				'server_url'		=> ['title'=>'服务器地址',	'type' =>'view',	'value'=>'<code>'.home_url().'/api/weapp/message/reply.json</code>'],
				'token'				=> ['title'=>'消息令牌',		'type'=>'text', 'class'=>'all-options'],
				'encodingaeskey'	=> ['title'=>'加密密钥',		'type'=>'text'],
			]],
			'minishop'				=> ['title'=>'小商店标准组件','type'=>'checkbox',	'description'=>'小程序已经开启小商店标准组件']
		];

		unset($feature_fields['contact']['options']['custom_reply']);

		if(!is_multisite()){
			unset($feature_fields['appid']);
		}

		$sections['features'] = ['title'=>'功能设置', 'fields'=>$feature_fields];
	}

	wpjam_register_option($option_name, ['sections'=>$sections,	'ajax'=>false,	'sanitize_callback'=>'weapp_option_sanitize_callback']);
}

function weapp_option_sanitize_callback($value){
	if(!is_multisite()){
		if(empty($value['appid'])){
			return new WP_Error('empty_appid', '小程序ID不能为空');
		}

		if(empty($value['secret'])){
			return new WP_Error('empty_secret', '小程序密钥不能为空');
		}

		if($value['appid']){
			if(!weapp_exists($value['appid'], $value['secret'])){
				return new WP_Error('invalid_appid', '小程序ID或者小程序密钥错误，请仔细检查后重新输入。');
			}
		}

		$appid	= $value['appid'];
	}else{
		$appid	= weapp_get_appid();
	}

	WEAPP_Account::create_table($appid);

	wpjam_register_activation(['WEAPP_Plugin', 'active_callback'], 'wp_loaded');

	return $value;
}

if(str_ends_with($GLOBALS['plugin_page'], '-users')){
	$object	= wpjam_get_user_signup_object('weapp');

	if($object){
		$object->register_bind_user_action();
	}
}