<?php
class WEAPP_Path{
	public static function get_config(){
		return ['page_keys'=>wpjam_get_page_keys('weapp', 'with_page')];
	}

	public static function path_callback($args, $name){
		if($name == 'custom'){
			return $args['custom'] ?: new WP_Error('empty_custom', '自定义路径不能为空。');
		}elseif($name == 'mini_program'){
			return [
				'type'	=> 'mini_program',
				'appid'	=> $args['appid'],
				'path'	=> $args['path'] ? '/'.ltrim($args['path'], '/') : ''
			];
		}elseif($name == 'contact'){
			return [
				'type'	=> 'contact',
				'tips'	=> $args['tips']
			];
		}elseif($name == 'tel'){
			return [
				'type'	=> 'tel',
				'phone'	=> $args['phone']
			];
		}
	}

	public static function get_sub_pages($base_menu, $appid){
		$subs	= [];

		if(wpjam_get_page_keys('weapp')) {
			if(weapp_has_funcscope(57, $appid)){
				$subs[$base_menu.'-search']	= [
					'menu_title'	=> '小程序搜索',	
					'function'		=> 'tab',
					'load_callback'	=> [self::class, 'plugin_page_load'],
				];
			}

			$subs[$base_menu.'-qrcodes']	= [
				'menu_title'	=> '路径和二维码',	
				'function'		=> 'form',
				'form_name'		=> 'generate_weapp_qrcode',
				'load_callback'	=> [self::class, 'plugin_page_load'],
			];
		}

		return $subs;
	}

	public static function submit_pages(){
		$last_submitted		= get_transient('weapp_submit_pages_last_submitted') ?: 0;

		if($last_submitted && time()-$last_submitted < DAY_IN_SECONDS){
			return new WP_Error('last_submitted', '24小时内已批量提交过。');
		}

		$page_key	= wpjam_get_data_parameter('page_key',['default'=>'']); 
		$offset		= (int)wpjam_get_data_parameter('offset');
		$number		= (int)wpjam_get_data_parameter('number');

		$pages		= [];
		$paths		= [];

		$pf_object	= wpjam_get_platform_object('weapp');
		$page_keys	= array_keys($pf_object->get_items());

		if($page_key){
			$index	= array_search($page_key, $page_keys);

			if(!$pf_object->get_item_arg($page_key, 'page_type')){
				$index	+=1;
			}

			$page_keys	= array_slice($page_keys, $index, -1);
		}

		if(!$page_keys){
			set_transient('weapp_submit_pages_last_submitted', time(), DAY_IN_SECONDS);
			wpjam_send_json();
		}

		$i = 0;

		foreach($page_keys as $page_key){
			if($pf_object->get_item_arg($page_key, 'page_type')){
				$type_paths	= $pf_object->get_paths($page_key, ['number'=>500, 'offset'=>$offset]);

				// print_r($page_key);
				// print_r($type_paths);

				foreach($type_paths as $path){
					$paths[]	= $path;
					$pages[]	= parse_url($path);

					$i++;
					$offset ++;

					if($i >= 500){
						break;
					}
				}
			}else{
				if($pf_object->get_fields($page_key)){
					continue;
				}

				$path	= $pf_object->get_path($page_key);

				if($path && !is_wp_error($path)){
					$paths[]	= $path;
					$pages[]	= parse_url($path);

					$i++;
				}
			}

			if($i >= 500){
				break;
			}

			$offset	= 0;
		}

		if($pages){
			$result	= weapp_submit_pages($pages);

			if(is_wp_error($result)){
				return $result;
			}
		}

		$number	+= $i;

		if(empty($pages) || $i < 500){
			set_transient('weapp_submit_pages_last_submitted', time(), DAY_IN_SECONDS);

			return ['errmsg'=>'所有'.$number.'个页面都提交完成。', 'data'=>implode('<br />', $paths).'<br />'];
		}else{
			$args	= http_build_query(['page_key'=>$page_key, 'offset'=>$offset, 'number'=>$number]);

			return ['done'=>0, 'errmsg'=>'批量提交中，请勿关闭浏览器，已提交了'.$number.'个页面。',	'args'=>$args, 'data'=>implode('<br />', $paths).'<br />'];
		}
	}

	public static function enable_search(){
		return weapp_delete_setting('search_status');
	}

	public static function search_site(){
		$keyword	= wpjam_get_data_parameter('keyword');
		$response	= weapp()->post('/wxa/sitesearch', compact('keyword'));

		if(is_wp_error($response)){
			return $response;
		}

		if(empty($response['items'])){
			return '搜索结果为空';
		}

		$result	= ''; 
		
		foreach($response['items'] as $item){
			$result	.= '<div class="search-result-item">
			<img src="'.$item['image'].'">
			<h3>'.$item['title'].'</h3>
			'.wpautop($item['description']."\n\n".'路径：'.$item['path']).'
			</div>';
		}

		return $result;
	}

	public static function search_image(){
		$img		= wpjam_get_data_parameter('img');

		add_filter('wpjam_thumbnail_args', function($args){
			return $args+['webp'=>0];
		});

		$file	= wpjam_download_image(wpjam_get_thumbnail($img));

		if(is_wp_error($file)){
			return $file;
		}

		$response	= weapp()->file('/wxa/imagesearch', $file, 'img');

		if(is_wp_error($response)){
			return $response;
		}

		if(empty($response['items'])){
			return '搜索结果为空';
		}

		$result	= ''; 

		foreach($response['items'] as $item){
			$result	.= '<div class="search-result-item">
			<img src="'.$item['image'].'">
			<h3>'.$item['title'].'</h3>
			'.wpautop($item['description']."\n\n".'路径：'.$item['path']).'
			</div>';
		}

		return $result;
	}

	public static function submit_post_page($post_id){
		if(defined('DOING_AUTOSAVE') && DOING_AUTOSAVE){
			return;
		}

		$pages	= [];

		$post_ids	= is_array($post_id) ? $post_id : [$post_id];
		$post_type	= get_post(current($post_ids))->post_type;
		$pf_object	= wpjam_get_platform_object('weapp');

		foreach(wpjam_get_page_keys('weapp', ['post_type'=>$post_type]) as $page_key){
			foreach($post_ids as $_post_id) {
				if(get_post($_post_id)->post_status != 'publish'){
					continue;
				}

				$path		= $pf_object->get_path($page_key, [$post_type.'_id'=>$_post_id]);
				$pages[]	= parse_url($path);
			}
		}

		if(empty($pages)){
			if(is_array($post_id)){
				return new WP_Error('empty_published_'.$post_type, '提交的内容都未发布。');
			}else{
				return;
			}
		}

		return weapp_submit_pages($pages);
	}

	public static function submit_term_page($term_id){
		$pages	= [];

		$term_ids	= is_array($term_id) ? $term_id : [$term_id];
		$taxonomy	= get_term(current($term_ids))->taxonomy;
		$query_key	= wpjam_get_taxonomy_query_key($taxonomy);
		$pf_object	= wpjam_get_platform_object('weapp');

		foreach(wpjam_get_page_keys('weapp', ['taxonomy'=>$taxonomy]) as $page_key){
			foreach($term_ids as $_term_id) {
				$path		= $pf_object->get_path($page_key, [$query_key=>$_term_id]);
				$pages[]	= parse_url($path);
			}
		}

		return weapp_submit_pages($pages);
	}

	public static function callback($data, $action_key, $submit_name){
		$data['path']	= wpjam_get_path('weapp', $data);

		if(is_wp_error($data['path'])){
			return $data['path'];
		}elseif(empty($data['path'])){
			return new WP_Error('invalid_page_key', '无效的页面');
		}

		if($submit_name == 'generate_qrcode'){
			return weapp_generate_qrcode_html($data);
		}elseif($submit_name == 'generate_url_link'){
			return weapp_generate_url_link_html($data['path']);
		}
	}

	public static function list_callback($id, $data, $action_key, $submit_name){
		return self::callback($data, $action_key, $submit_name);
	}

	public static function get_fields($action_key){
		if($action_key == 'submit_pages'){
			return [
				'page_key'	=> ['type'=>'hidden',	'value'=>''],
				'offset'	=> ['type'=>'hidden',	'value'=>0],
				'number'	=> ['type'=>'hidden',	'value'=>0],
			];
		}elseif($action_key == 'site_search'){
			return ['keyword'=>['type'=>'text',	'placeholder'=>'请输入关键字',	'requeired']];
		}elseif($action_key == 'image_search'){
			return ['img'=>['type'=>'img',	'item_type'=>'url',	'requeired']];
		}elseif($action_key == 'generate_weapp_qrcode'){
			return wpjam_get_path_fields('weapp', 'qrcode')+[
				'width'			=>['type'=>'number',	'title'=>'二维码宽度',	'description'=>'px',	'value'=>430,	'min'=>430,	'max'=>2000, 'step'=>10],
				'env_version'	=>['type'=>'select',	'title'=>'版本',	'options'=>['release'=>'正式版', 'trial'=>'体验版', 'develop'=>'开发版']],
				'color_set'		=>['type'=>'fieldset',	'title'=>'二维码颜色',	'fields'=>[
					'auto_color'	=>['type'=>'checkbox',	'value'=>1,	'description'=>'自动配置线条颜色'],
					'line_color'	=>['type'=>'color',		'show_if'=>['key'=>'auto_color',	'value'=>0]],
					'is_hyaline'	=>['type'=>'checkbox',	'description'=>'使用透明背景底色'],
				]],
			];
		}
	}

	public static function get_list_fields($id, $action_key=''){
		$fields		= [];
		$screen		= get_current_screen();
		$pf_object	= wpjam_get_platform_object('weapp');
		$page_keys	= [];
		
		if($screen->base == 'edit'){
			$post_type	= $screen->post_type;
			$query_key	= $post_type.'_id';
			$page_keys	= wpjam_get_page_keys('weapp', ['post_type'=>$post_type]);

			$fields['post_type']	= ['type'=>'hidden',	'value'=>$post_type];
		}elseif($screen->base == 'edit-tags'){
			$taxonomy	= $screen->taxonomy;
			$query_key	= wpjam_get_taxonomy_query_key($taxonomy);
			$page_keys	= wpjam_get_page_keys('weapp', ['taxonomy'=>$taxonomy]);

			$fields['taxonomy']	= ['type'=>'hidden',	'value'=>$taxonomy];
		}

		$fields[$query_key]	= ['type'=>'hidden',	'value'=>$id];

		$options	= [];
		$counter	= count($page_keys);

		foreach($page_keys as $page_key){
			$options[$page_key]	= $pf_object->get_item_arg($page_key, 'title');

			$show_if	= $counter > 1 ? ['key'=>'page_key','value'=>$page_key] : [];
			$path		= $pf_object->get_path($page_key, [$query_key=>$id]);

			$fields[$page_key.'_direct_path']	= ['title'=>'小程序直达路径',		'type'=>'view',	'show_if'=>$show_if,	'value'=>$path];

			if($redirect_path = weapp_get_redirect_path($path)){
				$fields[$page_key.'_redirect_path']	= ['title'=>'小程序中转路径',	'type'=>'view',	'show_if'=>$show_if,	'value'=>$redirect_path];
			}
		}			

		if(count($options) > 1){
			$fields	= ['page_key'=>['title'=>'小程序页面',	'type'=>'select',	'options'=>$options]]+$fields;
		}else{
			$fields	= ['page_key'=>['title'=>'小程序页面',	'type'=>'hidden',	'value'=>array_key_first($options)]]+$fields;
		}

		return array_merge($fields, ['width'=>['title'=>'二维码宽度',	'type'=>'number',	'description'=>'px',	'value'=>430,	'min'=>430,	'max'=>2000, 'step'=>10]]);
	}

	public static function builtin_page_load($screen){
		if(weapp_get_current_appid() && wpjam_get_page_keys('weapp')){
			if($screen->base == 'post'){
				if(weapp_has_funcscope(57) && wpjam_get_page_keys('weapp', ['post_type'=>$screen->post_type])){
					add_action('wp_after_insert_post',	[self::class, 'submit_post_page']);
				}
			}elseif($screen->base == 'edit'){
				if(wpjam_get_page_keys('weapp', ['post_type'=>$screen->post_type])){
					$submit_text	= ['generate_qrcode'	=>['text'=>'生成二维码',		'response'=>'append']];

					if(weapp_has_funcscope(88) && weapp_get_setting('grant_type') != 'single'){
						$submit_text['generate_url_link']	= ['text'=>'生成跳转链接',		'response'=>'append',	'class'=>''];
					}

					if(weapp_has_funcscope(57)){
						$submit_text['submit_post_page']	= ['text'=>'提交到微信搜索',	'callback'=>[self::class, 'submit_post_page'],	'class'=>''];
					}

					wpjam_register_list_table_action('weapp_action', [
						'title'			=> '小程序路径和二维码',
						'page_title'	=> '小程序路径和二维码',
						'post_status'	=> ['publish'],
						'fields'		=> [self::class, 'get_list_fields'],
						'callback'		=> [self::class, 'list_callback'],
						'submit_text'	=> $submit_text
					]);

					if(weapp_has_funcscope(57)){
						wpjam_register_list_table_action('submit_weapp_page', [
							'title'			=> '提交到微信搜索',
							'post_status'	=> ['publish'],
							'callback'		=> [self::class, 'submit_post_page'],
							'direct'		=> true,
							'bulk'			=> true,
							'row_action'	=> false
						]);
					}
				}
			}elseif($screen->base == 'edit-tags'){
				if(wpjam_get_page_keys('weapp', ['taxonomy'=>$screen->taxonomy])){
					$submit_text	= ['generate_qrcode'	=>['text'=>'生成二维码',	'response'=>'append']];

					if(weapp_has_funcscope(88) && weapp_get_setting('grant_type') != 'single'){
						$submit_text['generate_url_link']	= ['text'=>'生成跳转链接',	'response'=>'append',	'class'=>''];
					}

					if(weapp_has_funcscope(57)){
						$submit_text['submit_term_page']	= ['text'=>'提交到微信搜索',	'callback'=>[self::class, 'submit_term_page'],	'class'=>''];
					}

					wpjam_register_list_table_action('weapp_action', [
						'title'			=> '小程序路径和二维码',
						'page_title'	=> '小程序路径和二维码',
						'fields'		=> [self::class, 'get_list_fields'],
						'callback'		=> [self::class, 'list_callback'],
						'submit_text'	=> $submit_text
					]);

					if(weapp_has_funcscope(57)){
						wpjam_register_list_table_action('submit_weapp_page', [
							'title'			=> '提交到微信搜索',
							'callback'		=> [self::class, 'submit_term_page'],
							'direct'		=> true,
							'bulk'			=> true,
							'row_action'	=> false
						]);
					}
				}
			}
		}
	}

	public static function plugin_page_load($plugin_page){
		if(str_ends_with($plugin_page, '-search')){
			if(weapp_get_setting('search_status') == 'off'){
				wpjam_register_plugin_page_tab('enable',	[
					'title'			=> '小程序搜索',
					'function'		=> 'form',	
					'summary'		=> '重新启用之前请在小程序后台设置页面打开小程序搜索',
					'submit_text'	=> '重新启用',
					'response'		=> 'redirect',
					'fields'		=> [],
					'callback'		=> [self::class, 'enable_search']
				]);
			}else{
				wpjam_register_plugin_page_tab('submit_pages', 		[
					'title'			=> '批量提交',
					'function'		=> 'form',
					'form_name'		=> 'submit_pages',
					'summary'		=> '批量提交所有页面到微信搜索。',
					'submit_text'	=> '批量提交',
					'response'		=> 'append',
					'fields'		=> [self::class, 'get_fields'],
					'callback'		=> [self::class, 'submit_pages'],
				]);

				wpjam_register_plugin_page_tab('site_search',	[
					'title'			=> '页面搜索',
					'function'		=> 'form',
					'form_name'		=> 'site_search',
					'response'		=> 'append',
					'submit_text'	=> '搜索',
					'response'		=> 'append',
					'fields'		=> [self::class, 'get_fields'],
					'callback'		=> [self::class, 'search_site']
				]);

				// wpjam_register_page_action('image_search',	[
				// 	'response'		=> 'append',
				// 	'submit_text'	=> '搜索',
				// 	'response'		=> 'append',
				// 	'fields'		=> [self::class, 'get_fields'],
				// 	'callback'		=> [self::class, 'ajax_search_image'],
				// ]);

				wp_add_inline_style('list-tables', '
					.search-result-item{margin-bottom:20px; border-bottom:1px solid #ccc; padding-bottom:20px;}
					.search-result-item:last-child{border-bottom:none; padding-bottom:none;}
					.search-result-item img{width:94px; height:94px; float:left; margin:0 20px 0 0;}
					.search-result-item em.highlight{font-style:normal;}
				');
			}
		}elseif(str_ends_with($plugin_page, '-qrcodes')){
			$submit_text	= ['generate_qrcode'	=>['text'=>'生成二维码']];

			if(weapp_has_funcscope(88) && weapp_get_setting('grant_type') != 'single'){
				$submit_text['generate_url_link']	= ['text'=>'生成跳转链接',	'class'=>''];
			}

			weapp_register_path('custom',	[
				'title'		=> '自定义',
				'fields'	=> ['custom'=>['type'=>'text',	'placeholder'=>'请输入自定义路径： /pages/...',	'required']],
				'callback'	=> [self::class, 'path_callback'],
			]);

			wpjam_register_page_action('generate_weapp_qrcode',[
				'response'		=> 'append', 
				'submit_text'	=> $submit_text,
				'fields'		=> [self::class, 'get_fields'],
				'callback'		=> [self::class, 'callback'],
				'validate'		=> true,
			]);
		}
	}

	public static function init(){
		if(is_admin()){
			wpjam_add_admin_load([
				'base'		=> ['post', 'edit', 'edit-tags'],
				'callback'	=> [self::class, 'builtin_page_load']
			]);

			// wp_add_inline_style('list-tables', 'div.response{ padding: .7em 2em 1em; background:#fff; min-width: 255px; max-width: 520px; }');
		}

		weapp_register_path('mini_program',	[
			'title'		=> '跳转小程序',
			'path'		=> false,
			'group'		=> ['key'=>'external', 'title'=>'外部链接'],
			'callback'	=> [self::class, 'path_callback'],
			'fields'	=> [
				'appid'	=> ['type'=>'text',	'class'=>'all-options',		'required',		'placeholder'=>'请输入要跳转的小程序APPID'],
				'path'	=> ['type'=>'text',	'class'=>'regular-text',	'placeholder'=>'请输入要跳转小程序的路径，不填则跳转到首页。']
			]
		]);

		weapp_register_path('web_view',	[
			'title'		=> '公众号文章',
			'group'		=> 'external',
			'path'		=> false,
			'fields'	=> ['src'	=> ['type'=>'url',	'required',	'placeholder'=>'请输入关联的公众号文章的链接。']]
		]);

		weapp_register_path('contact',	[
			'title'		=> '进入客服对话框', 
			'group'		=> 'external',
			'path'		=> false,
			'callback'	=> [self::class, 'path_callback'],
			'fields'	=> ['tips'	=> ['type'=>'text',	'placeholder'=>'请输入跳转之前的提示语，不填则直接进入对话框。']]
		]);

		weapp_register_path('tel',	[
			'title'		=> '拨打电话', 
			'group'		=> 'external',
			'path'		=> false,
			'callback'	=> [self::class, 'path_callback'],
			'fields'	=> ['phone'	=> ['type'=>'text',	'placeholder'=>'请输入电话号码。']]
		]);
	}
}

if(is_weapp()){
	wpjam_register_config(['WEAPP_Path', 'get_config']);
}

function weapp_register_path($page_key, $args=[]){
	return WPJAM_Path::create($page_key, array_merge($args, ['path_type'=>'weapp']));
}

function weapp_get_path($page_key, $args=[]){
	return wpjam_get_path('weapp', $page_key, $args);
}

function weapp_get_redirect_path($path){
	$redirect_path	= apply_filters('weapp_redirect_path', '');

	if($redirect_path && parse_url($redirect_path, PHP_URL_PATH) != parse_url($path, PHP_URL_PATH)){
		return $redirect_path.urlencode($path);
	}

	return '';
}

function weapp_get_wxacode($args, $appid=''){
	$args	= wp_parse_args($args, [
		'type'			=> 'wxacode',
		'stream'		=> false,
		'return'		=> 'url',
		'auto_color'	=> true,
		'line_color'	=> '#000000'
	]);

	if($args['auto_color']){
		unset($args['line_color']);

		$args['auto_color']	= true;
	}else{
		unset($args['auto_color']);

		$line_color	= $args['line_color'] ?: '#000000';

		if(is_string($line_color)){
			list($r, $g, $b)	= sscanf($line_color, "#%02x%02x%02x");
			$args['line_color']	= compact('r', 'g', 'b');
		}
	}

	$method	= 'post';

	if($args['type'] == 'unlimit'){
		if(empty($args['page'])){
			return new WP_Error('empty_weapp_page', '小程序页面不能为空');
		}

		if(empty($args['scene'])){
			return new WP_Error('empty_weapp_scene', '小程序场景值不能为空');
		}

		$data	= wp_array_slice_assoc($args, ['page', 'scene', 'width', 'auto_color', 'line_color', 'is_hyaline', 'check_path', 'env_version']);
		$api	= '/wxa/getwxacodeunlimit';
	}else{
		if(empty($args['path'])){
			return new WP_Error('empty_weapp_path', '小程序路径不能为空');
		}

		if($args['type'] == 'wxacode'){
			$data	= wp_array_slice_assoc($args, ['path', 'width', 'auto_color', 'line_color', 'is_hyaline', 'env_version']);
			$api	= '/wxa/getwxacode';
		}elseif($args['type'] == 'qrcode'){
			$data	= wp_array_slice_assoc($args, ['path']);
			$api	= '/wxa/get_qrcode';	// 体验版二维码
			$method	= 'get';
		}else{
			return '';
		}
	}
	
	if($args['stream']){
		$media_id	= weapp($appid)->stream($api, array_merge($data, ['type'=>$args['type']]), $method);

		if(!is_wp_error($media_id) && $args['return'] == 'url'){
			return weapp($appid)->get_media_url($media_id, $args['type']);
		}

		return $media_id;
	}else{
		$weapp	= weapp($appid);

		if($args['type'] == 'wxacode'){
			$cache_key	= md5(maybe_serialize($data));
			$response	= $weapp->cache_get('wxacode:'.$cache_key);

			if($response !== false){
				return $response;
			}
		}

		$response	= call_user_func_array([$weapp, $method], [$api, $data, &$headers]);

		if($args['type'] == 'wxacode' && !is_wp_error($response)){
			if($headers['num-used'] != weapp_get_setting('wxacode_num_used', $appid)){
				weapp_update_setting('wxacode_num_used', $headers['num-used'], $appid);
			}

			$weapp->cache_set('wxacode:'.$cache_key, $response);
		}

		return $response;
	}	
}

function weapp_get_unlimit_wxacode($data, $appid=''){
	return wpjam_get_wxacode(array_merge($data, ['type'=>'unlimit']), $appid);
}

function weapp_create_qrcode($data, $return='url', $appid=''){
	$data['stream']	= true;
	$data['return']	= $return;

	return weapp_get_wxacode($data, $appid);
}

function weapp_generate_url_link($path, $args=[], $appid=''){
	$type	= wpjam_array_pull($args, 'type') ?: 'url_link';
	$query	= wpjam_array_pull($args, 'query') ?: '';

	if(strpos($path, '?') !== false){
		$parts	= explode('?', $path);
		$path	= $parts[0];
		$query	= $query ? wp_parse_args($query) : [];
		$query	= array_merge(wp_parse_args($parts[1]), $query);
	}

	if(is_array($query)){
		$query	= http_build_query($query);
	}

	if(strpos($query, '%') !== false){
		return '';
	}

	$expire	= [];

	if(!empty($args['expire_time']) || !empty($args['expire_interval'])){
		$expire['is_expire']	= true;

		if(!empty($args['expire_time'])){
			$expire['expire_type']		= 0;
			$expire['expire_time']		= (int)$args['expire_time'];
		}else{
			$expire['expire_type']		= 1;
			$expire['expire_interval']	= (int)$args['expire_interval'];
		}
	}

	$env_version	= wpjam_array_pull($args, 'env_version') ?: 'release';
	$data			= compact('path', 'query', 'env_version');

	if($type == 'url_link'){
		$key	= 'url_link';
		$data	= array_merge($data, $expire);
		$api	= '/wxa/generate_urllink';
	}elseif($type == 'scheme' || $type == 'nfc_scheme'){
		$key	= 'openlink';

		$data	= array_merge(['jump_wxa'=>$data], $expire);

		if($type == 'nfc_scheme'){
			foreach(['model_id', 'sn'] as $k){
				if(!empty($args[$k])){
					$data[$k]	= $args[$k];
				}
			}

			$api	= '/wxa/generatenfcscheme';
		}else{
			$api	= '/wxa/generatescheme';
		}
	}else{
		return '';
	}

	$response	= weapp($appid)->post($api, $data);

	if(is_wp_error($response)){
		if($response->get_error_code() == '40002'){
			weapp_update_setting('grant_type', 'single', $appid);
		}

		return $response;
	}

	return $response[$key];
}

function weapp_generate_scheme($path, $args=[], $appid=''){
	return weapp_generate_url_link($path, array_merge($args, ['type'=>'scheme']), $appid);
}

function weapp_generate_nfc_scheme($path, $args=[], $appid=''){
	return weapp_generate_url_link($path, array_merge($args, ['type'=>'nfc_scheme']), $appid);
}

function weapp_generate_short_link($page_url, $data=[], $appid=''){
	$data		= array_merge($data, ['page_url' => ltrim($page_url, '/')]);
	$response	= weapp($appid)->post('/wxa/genwxashortlink', $data);

	return is_wp_error($response) ? $response : $response['link'];
}

function weapp_query_url_link($url_link='', $appid=''){
	return weapp($appid)->post('/wxa/query_urllink', compact('url_link'));
}

function weapp_query_scheme($scheme='', $appid=''){
	return weapp($appid)->post('/wxa/queryscheme', compact('scheme'));
}

function weapp_submit_pages($pages, $appid=''){
	if(weapp_get_setting('search_status', $appid) == 'off'){
		return false;
	}

	$result	= weapp($appid)->post('/wxa/search/wxaapi_submitpages', compact('pages'));

	if(is_wp_error($result) && $result->get_error_code()=='85091'){
		wpjam_add_admin_notice([
			'key'		=> 'weapp_search_status_turned_off',
			'type'		=> 'error',
			'notice'	=> '小程序的搜索开关被关闭，请访问小程序后台设置页面打开开关再重试。',
		]);
		
		weapp_update_setting('search_status', 'off', $appid);
	}

	return $result;
}

function weapp_generate_qrcode_html($data, $appid=''){
	$data['width']	= $data['width'] ?? 430;
	$data['type']	= $data['type'] ?? 'wxacode';

	$path	= $raw_path	= $data['path'];

	if(isset($data['page_key']) && $data['page_key'] == 'live_room'){
		$data['path']	= $raw_path.'&type=9';
		$path			= $raw_path.'&type=10';
	}

	$wxacode	= weapp_get_wxacode($data, $appid);

	if(is_wp_error($wxacode)){
		return $wxacode;
	}

	$html	= '<p><strong style="word-break: break-all;">小程序直达路径：'.$path.'</strong></p>';
	$html	.= '<p><img src="'.$wxacode.'" style="width:200px;" /></p>';

	if($redirect_path = weapp_get_redirect_path($raw_path)){
		$data['path']	= $redirect_path;
		$wxacode		= weapp_get_wxacode($data, $appid);

		if(is_wp_error($wxacode)){
			return $wxacode;
		}

		$html	.= '<hr />';
		$html	.= '<p><strong style="word-break: break-all;">小程序中转路径：'.$redirect_path.'</strong></p>';
		$html	.= '<p><img src="'.$wxacode.'" style="width:200px;" /></p>';
	}

	$num_used	= weapp_get_setting('wxacode_num_used', $appid);

	if($num_used){
		$html	.= '<br /><p>小程序码用量：<strong>'.$num_used.' / 100,000</strong></p>';
	}

	return '<div style="min-height:300px;">'.$html.'</div>';
}

function weapp_generate_url_link_html($path, $appid=''){
	$urllink	= weapp_generate_url_link($path);

	if(is_wp_error($urllink)){
		return $urllink;
	}

	return '<div style="min-height:100px;">
	<p><strong style="word-break: break-all;">小程序跳转链接：'.$urllink.'</strong></p>
	</div>';
}


// 兼容
function weapp_generate_urllink($path, $query='', $args=[], $appid=''){
	$args['query']	= $query;

	return weapp_generate_url_link($path, $args, $appid);
}
