<?php
class WEAPP_Plugin{
	public static function __callStatic($method, $args){
		if(!in_array($method, ['list', 'apply', 'unbind', 'update'])){
			return;
		}

		$appid	= $args[1] ?? '';
		$weapp	= weapp($appid);

		if($method == 'list'){
			$data	= [];
			$force	= $args[0] ?? false;
			$list	= $weapp->cache_get('plugin_list');

			if($list !== false && !$force){
				return $list;
			}
		}else{
			$weapp->cache_delete('plugin_list');

			$data	= $args[0] ?? [];
			$keys	= ['plugin_appid'];

			if($method == 'apply'){
				$keys[]	= 'reason';
			}elseif($method == 'update'){
				$keys[]	= 'user_version';
			}

			$data	= wp_array_slice_assoc($data, $keys);
		}

		$data		= array_merge($data, ['action'=>$method]);
		$response	= $weapp->post('/wxa/plugin', $data);

		if($method == 'list' && !is_wp_error($response)){
			$list	= $response['plugin_list'];

			$weapp->cache_set('plugin_list', $list);

			return $list;
		}

		return $response;
	}

	public static function has($plugin_appid, $force=false, $appid=''){
		if(!weapp_has_funcscope(40, $appid)){
			return false;
		}

		$plugin_list	= self::list($force, $appid);
		$plugin_list	= is_wp_error($plugin_list) ? [] : $plugin_list;

		if(is_array($plugin_appid)){
			$has	= array_fill_keys($plugin_appid, false);

			foreach($plugin_list as $item){
				if(in_array($item['appid'], $plugin_appid)){
					$has[$item['appid']]	= $item;
				}
			}

			return $has;
		}else{
			foreach($plugin_list as $item){
				if($item['appid'] == $plugin_appid){
					return $item;
				}
			}

			return false;
		}
	}

	public static function get($plugin_appid){
		return self::has($plugin_appid, true) ?:  [];
	}

	public static function insert($data){
		$data['plugin_appid']	= $data['appid'];

		$result	= self::apply($data);

		return is_wp_error($result) ? $result : $data['appid'];
	}

	public static function delete($plugin_appid){
		return self::unbind(compact('plugin_appid'));
	}

	public static function query_items($count, $offset){
		$items	= self::list(true);

		if(is_wp_error($items)){
			return $items;
		}

		return ['items'=>$items, 'count'=>count($items)];
	}

	public static function render_item($item){
		$item['username']	= '<img src="'.$item['headimgurl'].'" width="50" />'.$item['nickname'];

		return $item;
	}

	Public static function get_actions(){
		return [
			'add'		=>['title'=>'申请'],
			'delete'	=>['title'=>'删除',	'direct'=>true,	'confirm'=>true]
		];		
	}

	public static function get_fields($action_key='', $appid=''){
		return [
			'username'	=> ['title'=>'插件',		'type'=>'view',		'show_admin_column'=>'only'],
			'appid'		=> ['title'=>'APPID',	'type'=>'text',		'show_admin_column'=>true],
			'status'	=> ['title'=>'状态',		'type'=>'view',		'show_admin_column'=>'only',	'options'=>[1=>'申请中', 2=>'申请通过', 3=>'已拒绝', 4=>'已超时']],
			'reason'	=> ['title'=>'理由',		'type'=>'textarea'],
		];
	}

	public static function get_sub_pages($base_menu, $appid){
		if(weapp_has_funcscope(40, $appid)){
			return [$base_menu.'-plugins'	=> [
				'menu_title'	=> '插件管理',	
				'function'		=> 'list',
				'title'			=> '插件',
				'singular'		=> 'weapp-plugin',
				'plural'		=> 'weapp-plugins',
				'model'			=> 'WEAPP_Plugin',
				'per_page'		=> 50,
				'primary_key'	=> 'appid'
			]];
		}
	}

	public static function active_callback(){
		$appid	= weapp_get_current_blog_appid();

		if($appid){
			$plugin_has	= self::has(['wx104a1a20c3f81ec2', 'wx34345ae5855f892d'], true, $appid);

			if(!$plugin_has['wx104a1a20c3f81ec2']){
				self::apply(['plugin_appid'=>'wx104a1a20c3f81ec2', 'reason'=>'启用企业微信客服'], $appid);
			}

			if(!empty($value['minishop'])){
				if(!$plugin_has['wx34345ae5855f892d']){
					self::apply(['plugin_appid'=>'wx34345ae5855f892d', 'reason'=>'启用小商店标准组件'], $appid);
				}

				$result	= weapp($appid)->post('/product/store/get_info');

				if(is_wp_error($result)){
					if($result->get_error_code() == '48001'){
						$errmsg	= '小程序未开启小商店标准组件，请到小程序后台开启后再来激活！';
					}else{
						$errmsg	= $result->get_error_message();
					}

					wpjam_add_admin_notice([
						'key'		=> 'weapp_minishop_unactive',
						'type'		=> 'error',
						'notice'	=> $errmsg,
					]);
				}
			}
		}
	}
}

class WEAPP_Sec{
	public static function filter_handle_upload_prefilter($file){
		if(is_admin()){
			return $file;
		}
		
		$capability	= is_multisite() ? 'manage_sites' : 'manage_options';

		if(current_user_can($capability)){
			return $file;
		}
		
		if(weapp_get_appid()){
			if(strpos($file['type'], 'image/') === 0){
				$result	=  weapp_img_sec_check($file['tmp_name']);

				if(is_wp_error($result)){
					$file['error']	= $result->get_error_message();
				}
			}
		}

		return $file;
	}

	public static function filter_pre_comment_approved($approved, $commentdata){
		if(is_wp_error($approved) || $approved == 0){
			return $approved;
		}

		if($commentdata['comment_content'] && weapp_get_appid()){
			$result	= weapp_msg_sec_check($commentdata['comment_content']);

			if(is_wp_error($result)){
				$errcode	= $result->get_error_code();

				if($errcode == '87014'){
					return new WP_Error($errcode, '你输入的内容违反了微信平台的规则，请修改后再提交。');
				}else{
					return new WP_Error($errcode, '错误码：'.$errcode.' | '.$result->get_error_message().' | 微信过滤系统暂不可用，请重试！');
				}
			}
		}

		return $approved;
	}

	public static function filter_pre_blacklist_check($pre, $text, $name='内容'){
		if(weapp_get_appid() && $text){

			$result	= weapp_msg_sec_check($text);

			if(is_wp_error($result)){
				$errcode	= $result->get_error_code();
				if($errcode == '87014'){
					return new WP_Error($errcode, '你输入的'.$name.'违反了微信平台的规则，请修改后再提交。');
				}else{
					return new WP_Error($errcode, '错误码：'.$errcode.' | '.$result->get_error_message().' | 微信过滤系统暂不可用，请重试！');
				}
			}
		}

		return $pre;
	}

	public static function init(){
		add_filter('pre_comment_approved',			[self::class, 'filter_pre_comment_approved'], 99, 2);
		add_filter('wp_handle_upload_prefilter',	[self::class, 'filter_handle_upload_prefilter']);
		add_filter('wpjam_pre_blacklist_check',		[self::class, 'filter_pre_blacklist_check'],10,3);
	}
}

class WEAPP_Webview_Domain{
	public static function __callStatic($method, $args){
		if($method == 'get'){
			$data	= [];
			$appid	= $args[0] ?? '';
		}else{
			$data	= $args[0];
			$appid	= $args[1] ?? '';
		}

		$data		= array_merge($data, ['action'=>$method]);
		$response	= weapp($appid)->post('/wxa/setwebviewdomain', $data);

		if($method == 'get' && !is_wp_error($response)){
			return $response['webviewdomain'];
		}

		return $response;
	}

	public static function get_sub_pages($base_menu, $appid){
		return [$base_menu.'-webview-domain'	=> [
			'menu_title'	=> '业务域名',	
			'function'		=> 'form',
			'form_name'		=> 'set_weapp_verify_txt',
			'load_callback'	=> [self::class, 'page_load']
		]];
	}

	public static function page_load(){
		$grant_type	= weapp_get_setting('grant_type');

		if($grant_type == 'single'){
			$webview_domain	= null;
		}else{
			$webview_domain	= self::get();
		}

		if($grant_type == 'single' || is_wp_error($webview_domain)){
			if($grant_type == 'single'){
				$summary	= '个人主体小程序不支持设置业务域名';
			}elseif($webview_domain->get_error_code() == '89231'){
				weapp_update_setting('grant_type', 'single');

				$summary	= '个人主体小程序不支持设置业务域名';
			}else{
				$summary	= ''.$webview_domain->get_error_message().'';
			}

			wpjam_register_page_action('set_weapp_verify_txt', [
				'submit_text'	=> '保存',
				'summary'		=> wpautop($summary),
				'submit_text'	=> '',
				'fields'		=> []
			]);
		}else{
			$summary	= '业务域名需经过ICP备案，新备案域名需24小时后才可配置。域名格式只支持英文大小写字母、数字及“- ”，不支持IP地址。<br/> 配置业务域名后，可打开任意合法的子域名，最多可以添加20个业务域名，一年只可修改50次业务域名。<br/> 请输入微信小程序后台配置业务域名界面提供的校验文件文件名称和内容，保存之后到微信小程序后台添加业务域名即可通过。';

			if($webview_domain){
				$summary	.= '现有业务域名：<br /><pre><code>'.implode('</code></pre> <pre><code>', $webview_domain).'</code></pre>';	
			}else{
				$summary	.= '暂未设置业务域名。';
			}

			$object	= wpjam_register_verify_txt('weapp_webview_domain', ['title'=>'微信小程序业务域名验证文件']);

			wpjam_register_page_action('set_weapp_verify_txt', [
				'summary'		=> wpautop($summary),
				'validate'		=> true,
				'submit_text'	=> '保存',
				'callback'		=> [$object, 'callback'],
				'fields'		=> [$object, 'get_fields']
			]);
		}
	}
}

class WEAPP_Minishop{
	public static function path_callback($args, $name){
		if($name == 'minishop_product'){
			if($product_id = (int)$args['product_id']){
				return 'plugin-private://wx34345ae5855f892d/pages/productDetail/productDetail?productId='.$product_id;
			}

			return new WP_Error('invalid_product_id', '无效的小商店商品ID');
		}
	}

	public static function init(){
		$appid	= weapp_get_current_blog_appid();

		if(!weapp_get_setting('minishop', $appid)){
			return;
		}

		weapp_register_path('minishop_home',	[
			'title'		=> '小商店 - 首页',
			'group'		=> ['key'=>'weapp_minishop',	'title'=>'微信小商店'],
			'path'		=> 'plugin-private://wx34345ae5855f892d/pages/home/home'
		]);

		weapp_register_path('minishop_cart',	[
			'title'		=> '小商店 - 购物车',
			'group'		=> 'weapp_minishop',
			'path'		=> 'plugin-private://wx34345ae5855f892d/pages/shoppingCart/shoppingCart'
		]);

		weapp_register_path('minishop_order_list',	[
			'title'		=> '小商店 - 订单列表页',
			'group'		=> 'weapp_minishop',
			'path'		=> 'plugin-private://wx34345ae5855f892d/pages/orderList/orderList?tabId=all'
		]);

		weapp_register_path('minishop_pending_pay_order_list',	[
			'title'		=> '小商店 - 待付款订单',
			'group'		=> 'weapp_minishop',
			'path'		=> 'plugin-private://wx34345ae5855f892d/pages/orderList/orderList?tabId=pendingPay'
		]);

		weapp_register_path('minishop_pending_recevied_order_list',	[
			'title'		=> '小商店 - 待收货订单',
			'group'		=> 'weapp_minishop',
			'path'		=> 'plugin-private://wx34345ae5855f892d/pages/orderList/orderList?tabId=pendingRecevied'
		]);

		weapp_register_path('minishop_after_sale_order_list',	[
			'title'		=> '小商店 - 退款订单',
			'group'		=> 'weapp_minishop',
			'path'		=> 'plugin-private://wx34345ae5855f892d/pages/orderList/orderList?tabId=afterSale'
		]);

		weapp_register_path('minishop_product',	[
			'title'		=> '小商店 - 商品详情页',
			'group'		=> 'weapp_minishop',
			'callback'	=> ['WEAPP_Minishop', 'path_callback'],
			'fields'	=> ['product_id'=>['type'=>'number',	'class'=>'all-options',	'required',	'placeholder'=>'请输入小商店商品ID']]
		]);
	}
}

function weapp_has_plugin($plugin_appid, $force=false, $appid=''){
	return WEAPP_Plugin::has($plugin_appid, $force, $appid);
}

function weapp_apply_plugin($plugin_appid, $reason, $appid=''){
	return WEAPP_Plugin::apply(compact('plugin_appid', 'reason'), $appid);
}

function weapp_msg_sec_check($content, $appid=''){
	return weapp($appid)->post('/wxa/msg_sec_check', compact('content'));
}

function weapp_img_sec_check($media, $appid=''){
	return weapp($appid)->file('/wxa/img_sec_check', $media);
}

function weapp_media_check_async($media_url, $media_type=2, $scene=1, $openid='', $appid=''){
	$version	= 2;
	return weapp($appid)->post('/wxa/media_check_async', compact('media_url', 'media_type'));
}