<?php
class WEAPP_Reply{
	public static function register_json($json){
		if(in_array($json, ['message.reply', 'weapp.message.reply',])){
			return wpjam_register_api($json, ['callback'=>[self::class, 'api_message_reply']]);
		}
	}

	public static function api_message_reply(){
		WEAPP_Message::delete_old();

		if(is_multisite()){
			if($weapp_settings	= WEAPP_Setting::get_by('blog_id', get_current_blog_id())){
				$appid	= $weapp_settings[0]['appid'];
			}else{
				$appid	= '';
			}
		}else{
			$appid	= weapp_get_appid();
		}

		if(empty($appid)){
			trigger_error("小程序未设置设置");
			exit;
		}

		$signature	= wpjam_get_parameter('signature',	['required'=>true]);
		$timestamp	= wpjam_get_parameter('timestamp',	['required'=>true]);
		$nonce		= wpjam_get_parameter('nonce',		['required'=>true]);

		$aes_key 	= base64_decode(weapp_get_setting('encodingaeskey', $appid).'=');
		$token		= weapp_get_setting('token', $appid);
		$crypt		= new WPJAM_Crypt([
			'method'		=> 'aes-256-cbc', 
			'options'		=> OPENSSL_ZERO_PADDING, 
			'block_size'	=> 32, 
			'key'			=> $aes_key, 
			'iv'			=> substr($aes_key, 0, 16),
			'token'			=> $token
		]);

		if($crypt->generate_weixin_signature($token, $timestamp, $nonce) == $signature){
			// 微信小程序第一次验证token
			if($echostr = wpjam_get_parameter('echostr')){
				echo $echostr;
				exit;
			}
		}else{
			trigger_error("签名验证错误");
			exit;
		}

		// 获取已加密的实体
		$encrypted_msg	= wpjam_json_decode(file_get_contents('php://input'));
		$encrypted_msg	= $encrypted_msg['Encrypt'];
		$msg_signature	= wpjam_get_parameter('msg_signature', ['required'=>true]);

		// 验证加密签名
		if($msg_signature != $crypt->generate_weixin_signature($token, $timestamp, $nonce, $encrypted_msg)){
			trigger_error("加密签名验证错误");
			exit;
		}

		try{
			$decrypted_msg	= $crypt->decrypt($encrypted_msg);
		}catch(Exception $e){
			trigger_error("AES 解密失败");
			exit;
		}

		if (strlen($decrypted_msg) < 16)	{
			echo '';
			exit;
		}

		//去除16位随机字符串,网络字节序和AppId
		$decrypted_msg	= $crypt->weixin_unpad($decrypted_msg, $from_appid);

		if($appid != $from_appid){
			trigger_error("AppId 校验错误\nfrom_appid: $from_appid	appid: $appid ");
			exit;
		}

		$decrypted_msg	= wpjam_json_decode($decrypted_msg);

		$require_reply	= true;
		$keyword		= '';
		$msg_type		= $decrypted_msg['MsgType'];
		if($msg_type == 'text'){
			$keyword	= $decrypted_msg['Content'];
		}elseif($msg_type == 'event'){
			$event		= $decrypted_msg['Event'];
			if($event == 'user_enter_tempsession'){
				$keyword	= $decrypted_msg['SessionFrom'];
			}elseif($event == 'kf_create_session' || $event == 'kf_close_session'){
				$require_reply	= false; 	// 无需发送客服消息
			}
		}

		$ToUserName		= $decrypted_msg['ToUserName'];
		$FromUserName	= $decrypted_msg['FromUserName'];

		$decrypted_msg	= wpjam_array_except($decrypted_msg, ['openid', 'appid']);

		WEAPP_Message::insert($decrypted_msg);

		echo "success";
		if($require_reply){
			$weapp_replies	= WEAPP_Reply_Setting::get_replies($msg_type, $keyword);
			$weapp_replies	= $weapp_replies ?: WEAPP_Reply_Setting::get_replies('default');

			if($weapp_replies){

				$i = 0;

				foreach ($weapp_replies as $weapp_reply) {
					$i++;

					$weapp_reply['reply']	= maybe_unserialize($weapp_reply['reply']);
					$result	= WEAPP_Reply_Setting::reply($FromUserName, $weapp_reply);

					if($result == 'transfer_customer_service'){
						echo wpjam_json_encode([
							'ToUserName'	=> $FromUserName,
							'FromUserName'	=> $ToUserName,
							'CreateTime'	=> time(),
							'MsgType'		=> 'transfer_customer_service',
						]);

						exit;
					}elseif(is_wp_error($result)){
						trigger_error(var_export($weapp_reply, true).$result->get_error_message());
					}

					if($msg_type == 'event'){
						if($i == 1) break;
					}else{
						if($i == 5) break;
					}
				}
			}else{
				echo wpjam_json_encode([
					'ToUserName'	=> $FromUserName,
					'FromUserName'	=> $ToUserName,
					'CreateTime'	=> time(),
					'MsgType'		=> 'transfer_customer_service',
				]);

				exit;
			}
		}

		exit;
	}

	public static function create_table(){
		WEAPP_Message::create_table();
		WEAPP_Reply_Setting::create_table();
	}

	public static function get_sub_pages($base_menu, $appid){
		$subs	= [];

		if(weapp_get_setting('contact', 'custom_reply')){
			$subs[$base_menu.'-replies']	= [
				'menu_title'		=> '自定义回复',
				'function'			=> 'list',
				'title'				=> '自定义回复',
				'singular'			=> 'weapp-reply',
				'plural'			=> 'weapp-replies',
				'primary_column'	=> 'msg_type',
				'search'			=> true,
				'model'				=> 'WEAPP_Reply_Setting'
			];

			$subs[$base_menu.'-messages']	= [
				'menu_title'		=> '消息管理',
				'function'			=> 'list',
				'title'				=> '消息',
				'singular'			=> 'weapp-message',
				'plural'			=> 'weapp-messages',
				'primary_column'	=> 'FromUserName',
				'model'				=> 'WEAPP_Message',
				'style'				=> '
				th.column-MsgType{width:28px;}
				th.column-username{width:260px;}
				th.column-CreateTime{width:84px;}
				'
			];
		}

		return $subs;
	}
}

class WEAPP_Message extends WPJAM_Model{
	use WEAPP_Trait;

	protected static $appid;

	public static function insert($data){
		$msg_type	= $data['MsgType'];

		if($msg_type == 'miniprogrampage'){
			$data['Content']	= $data['Title'];
			$data['AppId2']		= $data['AppId'];
			$data['PicUrl']		= $data['ThumbUrl'];
			$data['MediaId']	= $data['ThumbMediaId'];

			unset($data['Title']);
			unset($data['AppId']);
			unset($data['ThumbUrl']);
			unset($data['ThumbMediaId']);
		}

		unset($data['ToUserName']);

		$data['appid']	= self::get_appid();
		return parent::insert($data);
	}

	public static function delete_old(){
		if(!get_transient('weapp_messages_deleted')){
			set_transient('weapp_messages_deleted', 1, DAY_IN_SECONDS);
			return self::Query()->where_lt('CreateTime', time()-MONTH_IN_SECONDS)->delete();
		}
	}

	public static function reply_text($id, $data){
		return WEAPP_Reply_Setting::reply($data['touser'], $data);
	}

	public static function reply_image($id, $data){
		return WEAPP_Reply_Setting::reply($data['touser'], $data);
	}

	public static function reply_link($id, $data){
		return WEAPP_Reply_Setting::reply($data['touser'], $data);
	}

	public static function reply_miniprogrampage($id, $data){
		return WEAPP_Reply_Setting::reply($data['touser'], $data);
	}

	public static function query_items($limit, $offset){
		self::Query()->where('appid', self::get_appid());
		return parent::list($limit, $offset);
	}

	public static function get_actions(){
		$reply_types	= WEAPP_Reply_Setting::get_types();
		unset($reply_types['transfer_customer_service']);

		$actions	= [];
		foreach ($reply_types as $key => $title) {
			$actions['reply_'.$key]	= ['title'=>'回复'.$title];
		}

		return $actions;
	}

	public static function get_fields($action_key='', $id=''){
		if($action_key == ''){
			return [
				'username'		=> ['title'=>'用户',	'type'=>'text',	'show_admin_column'=>true],
				'Content'		=> ['title'=>'内容',	'type'=>'text',	'show_admin_column'=>true],
				'MsgType'		=> ['title'=>'类型',	'type'=>'text',	'show_admin_column'=>true, 'options'=>self::get_types()],
				'CreateTime'	=> ['title'=>'时间',	'type'=>'text',	'show_admin_column'=>true]
			];
		}else{
			$fields		= [];

			$message	= self::get($id);
			$touser		= $message['FromUserName'];
			$user		= WEAPP_User::render($touser);

			if($user){
				$fields['username'] = ['title'=>'用户',	'type'=>'view', 'value'=>$user['username']];
			}

			$reply_type	= str_replace('reply_', '', $action_key);

			if($reply_type == 'text') {
				$fields['reply']	= ['title'=>'内容',		'type'=>'textarea'];
			}elseif($reply_type == 'image') {
				$fields['reply']	= ['title'=>'图片',		'type'=>'img'];
			}elseif($reply_type == 'link') {
				$fields['reply']	= ['title'=>'图文链接', 	'type'=>'fieldset',		'fieldset_type'=>'array',	'fields'=>[
					'title'			=> ['title'=>'标题',	'type'=>'text'],
					'description'	=> ['title'=>'描述',	'type'=>'textarea'],
					'url'			=> ['title'=>'链接',	'type'=>'url'],
					'thumb_url'		=> ['title'=>'图片',	'type'=>'image'],
				]];
			}elseif($reply_type == 'miniprogrampage') {
				$fields['reply']	= ['title'=>'小程序卡片', 	'type'=>'fieldset',		'fieldset_type'=>'array',	'fields'=>[
					'title'			=> ['title'=>'标题',	'type'=>'text'],
					'pagepath'		=> ['title'=>'路径',	'type'=>'text'],
					'image'			=> ['title'=>'封面图',	'type'=>'img'],
				]];
			}

			$fields['reply_type']	= ['title'=>'',	'type'=>'hidden', 'value'=>$reply_type];
			$fields['touser']		= ['title'=>'',	'type'=>'hidden', 'value'=>$touser];

			return $fields;
		}
	}

	public static function render_item($item){
		$user	= WEAPP_User::render($item['FromUserName']);

		$item['username']	= ($user)?$user['username']:'';

		$MsgType	= $item['MsgType'];

		if($MsgType == 'text'){
			$item['Content']	= wpautop(wp_strip_all_tags($item['Content']));	// 过滤用户输入的 html 代码，防止攻击
		}elseif($MsgType == 'image'){
			$item['Content']	= '<img src="'.$item['PicUrl'].'" width="200" />';
		}elseif($MsgType == 'event'){
			if($item['Event'] == 'user_enter_tempsession'){
				$item['Content']	= '用户从 <strong>'.$item['SessionFrom'].'</strong> 进入会话';
			}elseif($item['Event'] == 'kf_create_session'){
				$item['Content']	= '用户从自定义回复转到客服工具，客服：'.$item['KfAccount'];
			}elseif($item['Event'] == 'kf_close_session'){
				$item['Content']= '结束客服，转回自定义回复，客服：'.$item['KfAccount'].'，结束类型：'.$item['CloseType'];
			}
		}

		$item['CreateTime']		= get_date_from_gmt(date('Y-m-d H:i:s',$item['CreateTime']));

		return $item;
	}

	public static function get_views(){
		global $wpdb, $current_admin_url, $plugin_page;

		$appid			= self::get_appid();
		$MsgTypes		= self::get_types();
		$MsgType		= ($_GET['MsgType'])??'';

		$total			= static::Query()->where('appid',$appid)->get_var('count(*)');
		$msg_counts 	= static::Query()->where('appid',$appid)->group_by('MsgType')->order_by('count')->get_results('COUNT( * ) AS count, `MsgType`');

		$views	= [];

		$class = empty($MsgType)? 'class="current"':'';
		$views['all'] = '<a href="'.$current_admin_url.'" '.$class.'>全部<span class="count">（'.$total.'）</span></a>';

		foreach ($msg_counts as $count) { 
			$class		= ($MsgType == $count['MsgType']) ? 'class="current"':'';
			$MsgType	= ($MsgTypes[$count['MsgType']])??$count['MsgType'];

			$views['msg-'.$count['MsgType']]	= '<a href="'.$current_admin_url.'&MsgType='.$count['MsgType'].'" '.$class.'>'.$MsgType.'<span class="count">（'.$count['count'].'）</span></a>';
		}

		return $views;
	}

	public static function get_types(){
		return [
			'text'				=>'文本',
			'event'				=>'事件',
			'image'				=>'图片',
			'miniprogrampage'	=>'小程序卡片'
		];
	}

	public static function get_handler(){
		return wpjam_get_handler([
			'table_name'		=> self::get_table(),
			'cache_group'		=> ['weapp_messages', true],
			'cache'				=> false,
			'field_types'		=> ['id'=>'%d'],
			'searchable_fields'	=> ['Content'],
			'filterable_fields'	=> ['MsgType'],
		]);
	}

	public static function get_table(){
		global $wpdb;
		return $wpdb->base_prefix . 'weapp_messages';
	}

	public static function create_table(){
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

		global $wpdb;

		$table = self::get_table();

		if($wpdb->get_var("show tables like '{$table}'") != $table) {
			$sql = "
			CREATE TABLE IF NOT EXISTS `{$table}` (
				`id` bigint(20) NOT NULL auto_increment,
				`appid` varchar(32) NOT NULL,
				`FromUserName` varchar(30)	NOT NULL,
				`CreateTime` int(10) NOT NULL,
				`MsgType` varchar(7) NOT NULL,
				`Content` text NOT NULL,
				`MsgId` bigint(20) NOT NULL,
				`PicUrl` varchar(500) NOT NULL,
				`MediaId` varchar(500) NOT NULL,
				`AppId2` varchar(32) NOT NULL,
				`PagePath` varchar(255) NOT NULL,
				`Event` varchar(50) NOT NULL,
				`SessionFrom` varchar(50) NOT NULL,
				`KfAccount` varchar(50) NOT NULL,
				`CloseType` varchar(15) NOT NULL,
				PRIMARY KEY	(`id`)
			) DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;
			";
	 
			dbDelta($sql);
		}
	}
}

class WEAPP_Reply_Setting extends WPJAM_Model{
	use WEAPP_Trait;

	protected static $appid;

	public static function get($id){
		$reply = parent::get($id);
		if (!$reply) {
			return $reply;
		}

		if (is_admin()) {
			if ($reply['msg_type'] == 'event') {
				$reply['sessionfrom'] = $reply['keyword'];
			}
		}

		$reply_type = ($reply['reply_type']) ?? 'text';

		$reply[$reply_type] = maybe_unserialize($reply['reply']);

		return $reply;
	}

	public static function insert($data){
		$data = self::prepare($data);

		return parent::insert($data);
	}

	public static function update($id, $data){
		$reply = self::get($id);
		if (!$reply) {
			return new WP_Error('reply_no_exists', '该自定义回复不存在');
		}

		$msg_type = $reply['msg_type'] ?? 'text';

		if (is_admin()) {
			$data['msg_type'] = $msg_type;
		}

		$data = self::prepare($data);

		$result = parent::update($id, $data);

		return $result;
	}

	public static function prepare($data){
		if (is_admin()) {

			$data['time'] = $data['time'] ?? time();

			$msg_type = $data['msg_type'];

			if ($msg_type == 'event') {
				$data['keyword'] = $data['sessionfrom'];
			}

			unset($data['sessionfrom']);

			$reply_type = $data['reply_type'] ?? 'text';

			$data['reply'] = ($reply_type == 'transfer_customer_service') ? '' : maybe_serialize($data[$reply_type]);
			$data['appid'] = static::get_appid();

			unset($data['text']);
			unset($data['image']);
			unset($data['link']);
			unset($data['miniprogrampage']);
		}

		return $data;
	}

	public static function delete($id){
		$reply	= self::get($id);

		if(!$reply){
			return new WP_Error('reply_no_exists', '该自定义回复不存在');
		}

		return parent::delete($id);
	}

	public static function reply($touser, $data){
		$reply_type = $data['reply_type'] ?? 'text';

		if ($reply_type == 'text') {
			$content = $data['reply'];
			if (!$content) {
				return false;
			}

			$reply_content = compact('content');
		} elseif ($reply_type == 'image') {
			if (empty($data['reply'])) {
				return new WP_Error('image_missing', '图片回复的图片字段为空');
			}

			$media_id = weapp_get_wp_img_media_id($data['reply']);
			if (is_wp_error($media_id)) {
				return $media_id;
			}

			$reply_content = compact('media_id');

		} elseif ($reply_type == 'link') {
			$link = $data['reply'];

			foreach (['title', 'description', 'url', 'thumb_url'] as $key) {
				if (empty($link[$key])) {
					return new WP_Error('link_' . $key . '_missing', '图文链接回复的' . $key . '字段为空');
				}
			}

			$reply_content = $link;
		} elseif ($reply_type == 'miniprogrampage') {
			$miniprogrampage = $data['reply'];

			foreach (['title', 'pagepath', 'image'] as $key) {
				if (empty($miniprogrampage[$key])) {
					return new WP_Error('miniprogrampage_' . $key . '_missing', '小程序卡片回复的' . $key . '字段为空');
				}
			}

			$media_id = weapp_get_wp_img_media_id($miniprogrampage['image']);
			if (is_wp_error($media_id)) {
				return $media_id;
			}

			$miniprogrampage['thumb_media_id'] = $media_id;
			unset($miniprogrampage['image']);

			$reply_content = $miniprogrampage;
		} elseif ($reply_type == 'transfer_customer_service') {
			return 'transfer_customer_service';
		}

		return weapp_send_custom_message([
			'touser'	=> $touser,
			'msgtype'   => $reply_type,
			$reply_type => $reply_content,
		]);
	}

	public static function get_replies($msg_type, $keyword = ''){
		$replies = [];
		if ($msg_type == 'image' || $msg_type == 'miniprogrampage' || $msg_type == 'default') {
			$replies = static::get_keywords($msg_type);
			if (!$replies) {
				return false;
			}
		} elseif ($msg_type == 'event') {
			$replies = static::get_keywords($msg_type);

			if ($replies && isset($replies[$keyword])) {
				$replies = $replies[$keyword];
			} else {
				return false;
			}
		} elseif ($msg_type == 'text') {
			$prefix_keyword = mb_substr($keyword, 0, 2);	// 前缀匹配，只支持2个字

			$replies		= static::get_keywords($msg_type, 'full');		// 完全匹配
			$replies_prefix = static::get_keywords($msg_type, 'prefix');	// 前缀匹配
			$replies_fuzzy  = static::get_keywords($msg_type, 'fuzzy');		// 模糊匹配

			$keyword = strtolower(trim($keyword));
			if ($replies && isset($replies[$keyword])) {
				$replies = $replies[$keyword];
			} elseif ($replies_prefix && isset($replies_prefix[$prefix_keyword])) {
				$replies = $replies_prefix[$prefix_keyword];
			} elseif ($replies_fuzzy && preg_match(
					'/' . implode('|', array_keys($replies_fuzzy)) . '/',
					$keyword,
					$matches
				)) {
				$fuzzy_keyword = $matches[0];
				$replies	   = $replies_fuzzy[$fuzzy_keyword];
			} else {
				return false;
			}
		}

		if (isset($_GET['debug'])) {
			print_r($replies);
		}

		return $replies;
	}

	public static function get_keywords($msg_type='text', $match=null){
		$results	= self::get_by('appid', static::get_appid());

		if(empty($results)){
			return $results;
		}

		if($match){
			$results	= array_filter($results, function($result) use ($msg_type, $match) {
				return $result['msg_type'] == $msg_type && $result['match'] == $match;
			});

			if(empty($results)){
				return $results;
			}
		}

		if ($msg_type == 'text' || $msg_type == 'event') {
			$replies = [];

			if ($results) {
				foreach ($results as $reply) {
					$key = strtolower(trim($reply['keyword']));
					if (strpos($key, ',')) {
						foreach (explode(',', $key) as $new_key) {
							$new_key = strtolower(trim($new_key));
							if ($new_key !== '') {
								$replies[$new_key][] = $reply;
							}
						}
					} else {
						$replies[$key][] = $reply;
					}
				}
			}
		} else {
			$replies = $results ?: [];
		}

		return $replies;
	}public static function query_items($limit, $offset){
		self::Query()->where('appid', parent::get_appid());

		return parent::list($limit, $offset);
	}

	public static function render_item($item){
		if ($item['reply_type'] == 'image') {
			$item['reply'] = '<img src="' . wpjam_get_thumbnail(
					wp_get_attachment_url($item['reply']),
					400
				) . '" width="200"/>';
		} elseif ($item['reply_type'] == 'link') {
			$reply = maybe_unserialize($item['reply']);

			$item['reply'] = '<div class="reply_item"><a target="_blank" href="' . $reply['url'] . '">';
			$item['reply'] .= '<h3>' . $reply['title'] . '</h3>';
			$item['reply'] .= '<div class="img_container small" style="background-image:url(' . wpjam_get_thumbnail(
					$reply['thumb_url'],
					['width' => 80, 'height' => 80, 'mode' => 1]
				) . ');"></div>';
			$item['reply'] .= '<p>' . $reply['description'] . '</p>';
			$item['reply'] .= '</a></div>';

		} elseif ($item['reply_type'] == 'miniprogrampage') {
			$reply = maybe_unserialize($item['reply']);

			$item['reply'] = '<div class="reply_item">
				<h3>' . $reply['title'] . '</h3>
				<div class="img_container big" style="background-image:url(' . wpjam_get_thumbnail(
					wp_get_attachment_url($reply['image']),
					['width' => 640, 'height' => 320, 'mode' => 1]
				) . '); background-size:320px 160px;">
				</div>
				<p>路径：' . $reply['pagepath'] . '</p>
			</div>';
		}

		if ($item['msg_type'] == 'text') {
			$matches	= self::get_matches(); 
			$match		= $matches[$item['match']] ?? $item['match'];
			$item['keyword']	.= '<br />'.$match.'匹配';
		} elseif ($item['msg_type'] == 'event') {
			$item['match'] = '';
		} else {
			$item['sessionfrom'] = '';
			$item['keyword']	 = '';
			$item['match']	   = '';
		}

		return $item;
	}

	public static function get_fields($action_key='', $id=0){
		$fields	= [
			'msg_type'		=> ['title'=>'消息类型',	'type'=>'radio',	'show_admin_column'=>true,	'options'=>WEAPP_Message::get_types()+['default'=>'默认']],
			'keyword'		=> ['title'=>'关键字', 	'type'=>'text',		'show_admin_column'=>true,		'show_if'=>['key'=>'msg_type', 'value'=>'text']],
			'match'			=> ['title'=>'匹配方式', 	'type'=>'radio',	'options'=>self::get_matches(),	'show_if'=>['key'=>'msg_type', 'value'=>'text']],
			'sessionfrom'	=> ['title'=>'会话参数',	'type'=>'text',		'show_if'=>['key'=>'msg_type', 'value'=>'event']],
			'reply_type' 	=> ['title'=>'回复类型',	'type'=>'radio', 	'show_admin_column'=>true,	'options'=>self::get_types()],
			'reply'			=> ['title'=>'回复内容',	'type'=>'view',		'show_admin_column'=>'only'],
			'text'			=> ['title'=>'内容',		'type'=>'textarea',	'show_if'=>['key'=>'reply_type', 'value'=>'text']],
			'image'			=> ['title'=>'图片',	 	'type'=>'img',		'show_if'=>['key'=>'reply_type', 'value'=>'image']],
			'link'			=> ['title'=>'图文链接', 	'type'=>'fieldset',	'show_if'=>['key'=>'reply_type', 'value'=>'link'],	'fieldset_type'=>'array',	'fields'=>[
				'title'			=> ['title'=>'标题', 	'type'=>'text'],
				'description'	=> ['title'=>'描述', 	'type'=>'textarea'],
				'url'			=> ['title'=>'链接', 	'type'=>'url'],
				'thumb_url'		=> ['title'=>'图片', 	'type'=>'image'],
			]],
			'miniprogrampage'=>['title'=>'小程序卡片', 'type'=>'fieldset','show_if'=>['key'=>'reply_type', 'value'=>'miniprogrampage'],	'fieldset_type'=>'array',	'fields'=>[
				'title'			=> ['title'=>'标题', 	'type'=>'text'],
				'pagepath'		=> ['title'=>'路径', 	'type'=>'text'],
				'image'			=> ['title'=>'封面图', 	'type'=>'img'],
			]],
			'status'		=> ['title'=>'状态',		'type'=>'checkbox',	'value'=> 1,'description'=>'激活'],
		];

		if($action_key == 'edit'){
			$weapp_reply = self::get($id);

			$fields['msg_type']['type'] = 'view';

			if ($weapp_reply['msg_type'] == 'text') {
				unset($fields['sessionfrom']);
			}elseif ($weapp_reply['msg_type'] == 'event') {
				unset($fields['keyword']);
				unset($fields['match']);
				$fields['sessionfrom']['value'] = $weapp_reply['keyword'];
			} else {
				unset($fields['keyword']);
				unset($fields['match']);
				unset($fields['sessionfrom']);
			}
		}

		return $fields;
	}

	public static function get_views(){
		global $wpdb, $current_admin_url, $plugin_page;

		$appid = static::get_appid();

		$msg_types = WEAPP_Message::get_types() + ['default' => '默认'];

		$msg_type = ($_GET['msg_type']) ?? '';
		$status   = ($_GET['status']) ?? 1;

		$total	  = static::Query()->where('appid', $appid)->where('status', 1)->get_var('count(*)');
		$status_0   = static::Query()->where('appid', $appid)->where('status', 0)->get_var('count(*)');
		$msg_counts = static::Query()->where('appid', $appid)->where('status', 1)->group_by('msg_type')->order_by(
			'count'
		)->get_results('COUNT( * ) AS count, `msg_type`');


		$views = [];

		$class		= (empty($msg_type) && $status) ? 'class="current"' : '';
		$views['all'] = '<a href="' . $current_admin_url . '" ' . $class . '>全部<span class="count">（' . $total . '）</span></a>';

		foreach ($msg_counts as $count) {
			$class	= ($msg_type == $count['msg_type']) ? 'class="current"' : '';
			$msg_type = ($msg_types[$count['msg_type']]) ?? $count['msg_type'];

			$views['msg-' . $count['msg_type']] = '<a href="' . $current_admin_url . '&msg_type=' . $count['msg_type'] . '" ' . $class . '>' . $msg_type . '<span class="count">（' . $count['count'] . '）</span></a>';
		}

		$class			 = empty($status) ? 'class="current"' : '';
		$views['status-0'] = '<a href="' . $current_admin_url . '&status=0" ' . $class . '>未激活<span class="count">（' . $status_0 . '）</span></a>';

		return $views;
	}

	public static function get_types(){
		return [
			'text'						=> '文本',
			'image'						=> '图片',
			'link'						=> '图文链接',
			'miniprogrampage'			=> '小程序卡片',
			'transfer_customer_service' => '转发到客服工具',
		];
	}

	public static function get_matches(){
		return [
			'full'		=> '完全',
			'prefix'	=> '前缀', 
			'fuzzy'		=> '模糊'
		];
	}

	public static function style(){
		?>
		<style type="text/css">
		th.column-match{
			width:70px;
		}

		th.column-reply_type, th.column-keyword{
			width: 120px;
		}

		th.column-msg_type{
			width:180px;
		}

		div.reply_item {
			padding: 12px;
			width: 320px;
			border: 1px solid #CCC;
			float: left;
			box-shadow: 1px 1px 3px #CCC;
			border-radius: 4px;
		}

		div.reply_item a {
			float: left;
			width: 320px;
		}

		div.reply_item div.small {
			width: 40px;
			height: 40px;
			float: right;
			background-size: cover;
		}

		div.reply_item div.big {
			width: 320px;
			height: 160px;
		}

		div.reply_item h3 {
			font-size: 14px;
			margin: 0 0 10px 0;
			overflow: hidden;
			text-overflow: ellipsis;
			display: -webkit-box;
			-webkit-box-orient: vertical;
			-webkit-line-clamp: 2;
		}

		div.reply_item p {
			margin: 0;
			float: left;
			width: 270px;
			overflow: hidden;
			text-overflow: ellipsis;
			display: -webkit-box;
			-webkit-box-orient: vertical;
			-webkit-line-clamp: 3;
		}
		</style>
		<?php
	}

	public static function get_handler(){
		return wpjam_get_handler([
			'table_name'		=> self::get_table(),
			'cache'				=> false,
			'group_cache_key'	=> 'appid',
			'cache_prefix'		=> self::get_appid(),
			'cache_group'		=> ['weapp_replies', true],
			'field_types'		=> ['id' => '%d', 'status' => '%d'],
			'searchable_fields'	=> ['keyword'],
			'filterable_fields'	=> ['msg_type', 'reply_type', 'status'],
		]);
	}

	public static function get_table(){
		global $wpdb;
		return $wpdb->base_prefix . 'weapp_replies';
	}

	public static function create_table(){
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

		global $wpdb;

		$wpdb->weapp_replies = $wpdb->base_prefix . 'weapp_replies';

		if ($wpdb->get_var("show tables like '{$wpdb->weapp_replies}'") != $wpdb->weapp_replies) {
			$sql = "
			CREATE TABLE IF NOT EXISTS `{$wpdb->weapp_replies}` (
				`id` bigint(20) NOT NULL auto_increment,
				`appid` varchar(32) NOT NULL,
				`keyword` varchar(255)	NOT NULL,
				`msg_type` varchar(7)	NOT NULL,
				`match` varchar(7) NOT NULL,
				`reply` text NOT NULL,
				`reply_type` varchar(31) NOT NULL,
				`status` int(1) NOT NULL,
				`time` int(10) NOT NULL,
				PRIMARY KEY	(`id`)
			) DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;
			";

			dbDelta($sql);
		}
	}
}

class_alias('WEAPP_Reply_Setting', 'WEAPP_ReplySetting');

// add_action('admin_head', ['WEAPP_Reply_Setting', 'style']);

