<?php
if(is_weapp()){
	wpjam_register_config('weapp_get_config');
}

// 创建 WEAPP 对象
function weapp($appid=''){
	$appid	= $appid ?: weapp_get_current_appid();

	if($appid){
		$setting	= weapp_get_setting($appid);

		if($setting){
			return WEAPP::get_instance($appid, $setting['secret']);
		}

		$wp_error	= new WP_Error('empty_weapp_setting', '请先在后台小程序设置中加入该小程序');
	}else{
		$wp_error	= new WP_Error('empty_appid', '如果测试环境，请在链接加上 ?appid=APPID');
	}

	if(wpjam_is_json_request()){
		trigger_error($wp_error->get_error_code());
		wpjam_send_json($wp_error);
	}elseif(!wp_doing_cron()){
		trigger_error($wp_error->get_error_code());
		wp_die('小程序设置信息为空',	'请先在后台小程序设置中加入该小程序');
	}
}

// 判断当前的 weapp 设置是否有效
function weapp_exists($appid, $secret){
	$weapp	= new WEAPP($appid, $secret);
	$result	= $weapp->get_access_token($force=true);
	return is_wp_error($result) ? false : true;
}

// 获取 appid
// 正式环境：		从 refer 获取
// 如果在后台：	从 $plugin_page 中通过正则获取
// 测试环境：		从 $_GET['appid'] 中获取
function weapp_get_current_appid(){
	if(!is_multisite()){
		return wpjam_get_setting('wpjam_weapp', 'appid');
	}

	$appid	= wpjam_get_current_var('weapp_appid');

	if(is_null($appid)){
		$appid	= '';
		$refer	= $_SERVER['HTTP_REFERER'] ??'';

		if($refer && preg_match('|https://servicewechat.com/(.*?)/.*?/|i', $refer, $matches)){
			if($matches[1] != 'touristappid'){
				$appid	= $matches[1];	
			}
		}elseif(is_admin()){
			$weapp_settings	= WEAPP_Setting::get_by('blog_id', get_current_blog_id());

			if(!$weapp_settings){
				return '';
			}

			if(count($weapp_settings) == 1){
				$appid	= $weapp_settings[0]['appid'];
			}else{
				$appid_str = '';

				if(!empty($GLOBALS['plugin_page'])){
					$appid_str	= $GLOBALS['plugin_page']; 
				}elseif(!empty($_REQUEST['page'])){
					$appid_str	= $_REQUEST['page'];
				}elseif(!empty($_REQUEST['plugin_page'])){
					$appid_str	= $_REQUEST['plugin_page'];
				}
				
				if($appid_str && preg_match('/[-|_](wx[A-Za-z0-9]{15,17})/', $appid_str, $matches)){
					$appid	= ($matches[1]);
				}else{
					return '';
				}
			}
		}else{
			if(!empty($_GET['appid'])){
				$query_appid	= $_GET['appid'];
			}else{
				$query_appid	= $GLOBALS['wp']->query_vars['appid'] ?? '';
			}

			if($query_appid && strpos($query_appid, 'wx') === 0){
				$appid = $query_appid;
			}
		}

		if($appid){
			weapp_set_current_appid($appid);
		}
	}

	return $appid;
}

function weapp_set_current_appid($appid){
	wpjam_set_current_var('weapp_appid', $appid);
}

function weapp_get_appid(){
	return weapp_get_current_appid();
}

function weapp_set_appid($appid){
	weapp_set_current_appid($appid);
}

function weapp_get_current_blog_appid(){
	if(is_multisite()){
		if(is_weapp()){
			return weapp_get_appid();
		}else{
			$settings	= WEAPP_Setting::get_by('blog_id', get_current_blog_id());

			if($settings){
				return current($settings)['appid'];
			}else{
				return '';
			}
		}
	}else{
		return weapp_get_appid();
	}
}

function weapp_get_version(){
	$version	= wpjam_get_current_var('weapp_version');

	if(is_null($version)){
		$refer	= $_SERVER['HTTP_REFERER'] ?? '';

		if($refer && preg_match('|https://servicewechat.com/.*?/(.*?)/|i', $refer, $matches)){
			$version	= $matches[1];
		}else{
			$version	= 0;
		}

		wpjam_set_current_var('weapp_version', $version);
	}

	return $version;
}

function weapp_setting($appid=''){
	if(is_multisite()){
		$appid	= $appid ?: weapp_get_current_appid();
		$data	= $appid ? WEAPP_Setting::get($appid) : null;

		if(!$data){
			return null;
		}

		$blog_id	= $data['blog_id'];
		$option		= 'weapp_'.$appid;
	}else{
		$option		= 'wpjam_weapp';
		$blog_id	= null;
	}

	return wpjam_setting('option', $option, $blog_id);
}

// weapp_get_setting()
// weapp_get_setting($appid)
// weapp_get_setting($name)
// weapp_get_setting($name, $appid)
function weapp_get_setting(...$args){
	$args_num	= count($args);
	$appid		= '';

	if($args_num == 0){
		$name	= '';
	}elseif($args_num == 1){
		$name	= $args[0];
		
		if($name && preg_match('/(wx[A-Za-z0-9]{15,17})/', $name)){
			$appid	= $name;
			$name	= '';
		}
	}elseif($args_num >= 2){
		$name	= $args[0];
		$appid	= $args[1];
	}

	$object	= weapp_setting($appid);

	if($object){
		if(is_multisite()){
			$appid	= $appid ?: weapp_get_current_appid();
			$values	= array_merge(WEAPP_Setting::get($appid), $object->get_option());

			return $name ? ($values[$name] ?? null) : $values;
		}else{
			return $object->get_setting($name);
		}
	}

	return $name ? null : [];
}

function weapp_update_setting($name, $value, $appid=''){
	$object	= weapp_setting($appid);

	return $object ? $object->update_setting($name, $value) : false;
}

function weapp_delete_setting($name, $appid=''){
	$object	= weapp_setting($appid);

	return $object ? $object->delete_setting($name) : false;
}

function weapp_get_settings($blog_id=''){
	if(is_multisite()){
		$blog_id	= $blog_id ?: get_current_blog_id();
		$settings	= WEAPP_Setting::get_by('blog_id', $blog_id);

		foreach($settings as &$setting){
			$setting_ex	= get_blog_option($setting['blog_id'], 'weapp_'.$setting['appid']) ?: [];
			$setting	= array_merge($setting, $setting_ex);
		}

		return $settings;
	}else{
		return [weapp_get_setting()];
	}
}

function weapp_has_funcscope($scope=0, $appid=''){
	$appid	= $appid ?: weapp_get_current_appid();

	if($appid && weapp_get_setting('component_blog_id', $appid)){
		$default	= false;
	}else{
		$default	= true;
	}

	return apply_filters('weapp_has_funcscope', $default, $scope, $appid);;
}

function weapp_get_config(){
	$config		= ['version'=>weapp_get_version()];
	$contact	= weapp_get_setting('contact') ?: 'weapp';
	$minishop	= (int)weapp_get_setting('minishop');

	$config['contact']	= ['type'=>$contact];

	if($contact == 'work_weixin'){
		$config['contact']['plugid']	= weapp_get_setting('work_weixin_plugid') ?: '';
	}

	if($minishop){
		$config['minishop']	= $minishop;
	}

	return $config;
}


// 获取当前的 openid
// 如果 $code 不为空，从 jscode 接口中获取
// 否则从 $access_token 中获取
function weapp_get_current_openid(){
	if(!is_weapp()){
		return new WP_Error('illegal_platform', '该函数只能在小程序中调用');
	}

	$openid	= wpjam_get_current_var('weapp_openid');

	if(is_null($openid)){
		$access_token	= wpjam_get_parameter('access_token');

		if(!$access_token){
			if((isset($_GET['debug']) || isset($_GET['debug_openid'])) && isset($_GET['openid'])){
				return $_GET['openid'];	// 用于测试
			}else{
				return new WP_Error('illegal_access_token', 'Access Token 为空！');
			}
		}

		$appid	= weapp_get_current_appid();
		$openid = WEAPP_Account::get_openid_by_access_token($appid, $access_token);

		if(!is_wp_error($openid)){
			weapp_set_current_openid($openid);
		}
	}

	return $openid;
}

function weapp_set_current_openid($openid){
	wpjam_set_current_var('weapp_openid', $openid);
}

function weapp_get_user_object($openid, $appid=''){
	$appid	= $appid ?: weapp_get_current_appid();

	return WEAPP_Account::get_instance($appid, $openid);
}

function weapp_get_user($openid, $appid=''){
	$appid	= $appid ?: weapp_get_current_appid();

	return WEAPP_Account::get($appid, $openid);
}

// 获取 $session_key
// 如果 $code 不为空，从 jscode2session 接口中获取，并存到内存
// 否则获取当前 openid，并且内存中获取
function weapp_get_session_key($code=''){
	if($code){
		$session	= weapp()->jscode2session($code);
	}else{
		$openid		= weapp_get_current_openid();

		if(is_wp_error($openid)){
			return $openid;
		}

		$session	= weapp()->get_session_by_openid($openid);
	}

	if(is_wp_error($session)){
		return $session;
	}

	return $session['session_key'];
}

// 小程序分类
function weapp_get_category($force=false, $appid=''){
	$weapp	= weapp($appid);

	$category_list	= $weapp->cache_get('category_list');

	if($category_list === false || $force){
		$response = $weapp->get('/wxa/get_category');

		if(is_wp_error($response)){
			return $response;
		}

		$category_list	= $response['category_list'];

		$weapp->cache_set('category_list', $category_list);
	}

	return $category_list;
}


// 发送客服消息
function weapp_send_custom_message($data, $appid=''){
	return weapp($appid)->post('/cgi-bin/message/custom/send', $data);
}

function weapp_upload_media($media, $type='image', $appid=''){
	return weapp($appid)->file('/cgi-bin/media/upload?type='.$type, $media, 'media', 10);
}

function weapp_get_material($media_id, $appid=''){
	$weapp		= weapp($appid);
	$data		= ['media_id'=>$media_id, 'type'=>'material'];
	$response	= $weapp->stream('/cgi-bin/material/get_material', $data);

	if(is_wp_error($response)){
		$media_file	= $weapp->get_media_file($media_id, $type);

		if($response->get_error_code() == '40007'){	//  invalid media_id
			$im = imagecreatetruecolor(120, 20);
			$text_color = imagecolorallocate($im, 233, 14, 91);
			imagestring($im, 1, 5, 5,  'invalid media_id', $text_color);

			imagejpeg($im, $media_file, 100 ); // 存空图片，防止重复请求
		}

		return $response;
	}

	return $weapp->get_media_url($media_id);
}

// 将WP的图片上传到小程序临时素材
function weapp_get_wp_img_media_id($img_id, $appid=''){
	if(!get_post($img_id)){
		return new WP_Error('invalid_attachment_id', '非法附件ID');
	}

	$media	= get_post_meta($img_id, 'weapp_media', true);

	if(!$media  || ((time() - $media['created_at'] + 600) > DAY_IN_SECONDS*3)){

		$img_file	= get_attached_file($img_id);

		if(!$img_file){
			return new WP_Error('invalid_attachment_id', '非法附件ID');
		}

		$media	= weapp_upload_media($img_file, 'image', $appid);

		if(is_wp_error($media)){
			return $media;
		}

		update_post_meta($img_id, 'weapp_media', $media);
	}

	return $media['media_id'];
}

function weapp_user_signup($openid, $args=[]){
	$object	= wpjam_get_user_signup_object('weapp');

	return $object ? call_user_func([$object, 'signup'], $openid, $args) : null;
}

function weapp_user_bind($openid, $user_id){
	$object	= wpjam_get_user_signup_object('weapp');

	return $object ? call_user_func([$object, 'bind'], $openid, $user_id) : null;
}
