<?php
class WEAPP_Tabbar_Extend{
	public static function get_sub_pages($base_menu, $appid){
		if(wpjam_get_tabbar('weapp')){
			return [$base_menu.'-tabbar'	=> [
				'menu_title'	=> '底部菜单栏',	
				'function'		=> 'tab',
				'tabs'			=> [
					'setting'	=> [
						'title'			=> '基础设置',
						'function'		=> 'option',
						'option_name'	=> 'weapp_tabbar_'.$appid,
						'model'			=> 'WEAPP_Tabbar'
					],
					'list'		=> [
						'title'			=> '菜单栏',
						'function'		=> 'list',
						'plural'		=> 'tabbar_items',
						'singular'		=> 'tabbar_item',
						'primary_key'	=> 'page_key',
						'model'			=> 'WEAPP_Tabbar_Items',
						'fixed'			=> false
					]
				]
			]];
		}
	}
}

class WEAPP_Tabbar extends WPJAM_Option_Model{
	public static function get_fields(){
		$fields		= [
			'color'				=> ['title'=>'文字默认颜色',	'type'=>'color'],
			'selectedColor'		=> ['title'=>'文字选中颜色',	'type'=>'color'],
			'backgroundColor'	=> ['title'=>'背景颜色',		'type'=>'color'],
			'borderStyle'		=> ['title'=>'上边框颜色',	'type'=>'select',	'options'=>['black'=>'黑色','white'=>'白色']],
		];

		$icon_dir	= self::get_setting('icon_dir');
		$icon_url	= self::get_setting('icon_url');

		if($icon_url && $icon_dir && ($icon_handle = opendir($icon_dir))){
			while(($style = readdir($icon_handle)) !== false){
				if($style == '.' || $style == '..' || is_file($icon_dir.$style)){
					continue;
				}

				if(file_exists($icon_dir.$style.'/'.$style.'.png')){
					$styles[$style]	= '<img src="'.$icon_url.$style.'/'.$style.'.png" width="220" height="30" />';
				}else{
					$styles[$style]	= $style;
				}
			}   

			closedir($icon_handle);

			if($styles){
				ksort($styles);

				$fields['icon_style']	= ['title'=>'图标样式',	'type'=>'radio',	'sep'=>'<br />',	'options'=>$styles+['custom'=>'自定义图标']];
			}
		}

		return $fields;
	}

	public static function filter_option($value){
		$value		= $value ?: [];
		$defaults	= apply_filters('weapp_tabbar_defaults', [], weapp_get_current_appid());

		return wp_parse_args($value, $defaults);
	}

	public static function get_items(){
		$key	= (int)weapp_get_version() ? 'released' : 'trial';
		$items	= self::get_setting($key, []);

		if(empty($items)){
			return null;
		}

		$current	= self::get_setting('list', []);
		$icon_style	= self::get_setting('icon_style', 'custom');
		$icon_dir	= self::get_setting('icon_dir');
		$icon_url	= self::get_setting('icon_url');

		if($icon_style != 'custom' && $icon_dir && $icon_url && is_dir($icon_dir.$icon_style)){
			$icon_url	= $icon_url.$icon_style;
		}else{
			$icon_url	= '';
		}

		foreach($items as &$item){
			$page_key	= $item['page_key'] ?? '';

			if($page_key){
				$setting	= $current[$page_key] ?? [];

				if($setting){
					if(!empty($setting['text'])){
						$item['text']	= $setting['text'];
					}

					if(!empty($setting['url'])){
						$item['url']	= $setting['url'];
					}
				}

				if($icon_url){
					$tabbar_item	= wpjam_get_tabbar('weapp', $page_key);
					$icon_key		= $tabbar_item['icon_key'] ?? $page_key;

					$item['iconPath']			= wpjam_get_thumbnail($icon_url.'/'.$icon_key.'.png');
					$item['selectedIconPath']	= wpjam_get_thumbnail($icon_url.'/'.$icon_key.'-selected.png');
				}else{
					$item['iconPath']			= !empty($setting['iconPath']) ? wpjam_get_thumbnail($setting['iconPath'], '81x81') : ($item['iconPath'] ?? '');
					$item['selectedIconPath']	= !empty($setting['selectedIconPath']) ? wpjam_get_thumbnail($setting['selectedIconPath'], '81x81') : ($item['selectedIconPath'] ?? '');
				}
			}
		}

		$parsed	= wpjam_array_except(self::get_setting(), ['icon_style', 'icon_dir', 'icon_url', 'released', 'trial']);

		$parsed['list']	= array_values($items);

		return $parsed;
	}

	public static function get_icon_url(){
		$icon_style		= self::get_setting('icon_style', 'custom');

		if($icon_style != 'custom'){
			$icon_url	= self::get_setting('icon_url');
			$icon_dir	= self::get_setting('icon_dir');

			if($icon_url && $icon_dir &&  is_dir($icon_dir)){
				return $icon_url.$icon_style;
			}
		}

		return '';
	}
}

class WEAPP_Tabbar_Items extends WPJAM_Model{
	public static function get($page_key){
		$item	= parent::get($page_key);

		if($item){
			$tabbar_item	= wpjam_get_tabbar('weapp', $page_key);
			
			return array_merge($item, ['name'=>$tabbar_item['text']]);
		}

		return $item;
	}

	public static function update($page_key, $data){
		$tabbar	= wpjam_get_tabbar('weapp', $page_key);

		if(!empty($tabbar['webview'])){
			$data['url']	= $data['url'] ?? '';

			if(empty($data['url'])){
				return new WP_Error('empty_url', '链接不能为空');
			}

			$data['url']	= set_url_scheme($data['url'], 'https');

			$webview_domain	= WEAPP_Webview_Domain::get();

			if(is_wp_error($webviewdomain)){
				return $webviewdomain;
			}

			$domain	= 'https://'.parse_url($data['url'], PHP_URL_HOST);

			if(!in_array($domain, $webviewdomain)){
				$result	= WEAPP_Webview_Domain::add(['webviewdomain'=>$domain]);

				if(is_wp_error($result)){
					$errcode	= $result->get_error_code();

					if($errcode == '89019'){
						// 业务域名无更改，无需重复设置
					}elseif($errcode == '89020'){
						return new WP_Error($errcode, '尚未设置小程序业务域名，请先在第三方平台中设置小程序业务域名后在调用本接口');
					}elseif($errcode == '89021'){
						return new WP_Error($errcode, '请求保存的域名不是第三方平台中已设置的小程序业务域名或子域名');
					}elseif($errcode == '89029'){
						return new WP_Error($errcode, '业务域名数量超过限制');
					}elseif($errcode == '89031'){
						return new WP_Error($errcode, '业务域名数量超过限制');
					}else{
						return $result;
					}
				}
			}
		}

		return parent::update($page_key, $data);
	}

	public static function get_handler(){
		$appid		= weapp_get_current_appid();
		$option		= 'weapp_tabbar_'.$appid;
		$handler	= wpjam_get_handler($option);

		if(!$handler){
			$update_required	= false;

			$tabbar	= wpjam_get_tabbar('weapp');
			$items	= WEAPP_Tabbar::get_setting('list') ?: [];

			foreach($items as $page_key => $tabbar_item){
				if(isset($tabbar[$page_key])){
					unset($tabbar[$page_key]);
				}else{
					unset($items[$page_key]);
					$update_required	= true;
				}
			}

			if($tabbar){
				foreach($tabbar as $page_key => $tabbar_item){
					$text	= $tabbar_item['text'];

					$items[$page_key]	= compact('page_key', 'text');
				}

				$update_required	= true;
			}

			if($update_required){
				WEAPP_Tabbar::update_setting('list', $items);
			}
		}

		return wpjam_get_handler([
			'primary_key'	=> 'page_key',
			'primary_title'	=> '页面',
			'option_name'	=> $option,
			'items_field'	=> 'list',
		]);
	}

	public static function get_actions(){
		return  [
			// 'add'	=> ['title'=>'新增'],
			'edit'	=> ['title'=>'设置'],
			// 'delete'=> ['title'=>'删除',	'direct'=>true,	'confirm'=>true,	'bulk'=>true],
		];
	}

	public static function render_item($item){
		$page_key		= $item['page_key'];
		$tabbar_item	= wpjam_get_tabbar('weapp', $page_key);
		
		$item['name']		= $tabbar_item['text'];
		$item['pagePath']	= $item['pagePath'] ?? weapp_get_path($page_key);
		$item['apply_for']	= [];

		foreach(['trial'=>'体验版', 'released'=>'正式版'] as $version => $name){
			$page_keys	= array_column(WEAPP_Tabbar::get_setting($version, []), 'page_key');

			if($page_keys && in_array($page_key, $page_keys)){
				$item['apply_for'][]	= $name;
			}
		}

		$item['apply_for']	= implode(' ', $item['apply_for']);

		if($icon_url = WEAPP_Tabbar::get_icon_url()){
			$icon_key	= $tabbar_item['icon_key'] ?? $page_key;

			$item['iconPath']			= $icon_url.'/'.$icon_key.'.png';
			$item['selectedIconPath']	= $icon_url.'/'.$icon_key.'-selected.png';
		}else{
			$item['iconPath']			= !empty($item['iconPath']) ? wpjam_get_thumbnail($item['iconPath'], '81x81') : '';
			$item['selectedIconPath']	= !empty($item['selectedIconPath']) ? wpjam_get_thumbnail($item['selectedIconPath'], '81x81') : '';
		}

		$item['iconPath']			= !empty($item['iconPath']) ? '<img src="'.$item['iconPath'].'" width="40" />' : '';
		$item['selectedIconPath']	= !empty($item['selectedIconPath']) ? '<img src="'.$item['selectedIconPath'].'" width="40" />' : '';

		return $item;
	}

	public static function get_fields($action_key='', $page_key=''){
		$fields	= [
			'name'		=> ['title'=>'名称',		'type'=>'view',	'show_admin_column'=>true],
			'text'		=> ['title'=>'按钮文字',	'type'=>'text',	'show_admin_column'=>true,	'class'=>'all-options',	'requried'],
			'iconPath'			=> ['title'=>'默认图标',	'type'=>'img',	'show_admin_column'=>true,	'item_type'=>'url',	'size'=>'81x81'],
			'selectedIconPath'	=> ['title'=>'选中图标',	'type'=>'img',	'show_admin_column'=>true,	'item_type'=>'url',	'size'=>'81x81'],
			'pagePath'			=> ['title'=>'页面路径',	'type'=>'view',	'show_admin_column'=>'only']
		];

		if(WEAPP_Tabbar::get_icon_url()){
			$fields['iconPath']['show_admin_column']	= $fields['selectedIconPath']['show_admin_column']	= 'only';
		}

		if($action_key && $page_key){
			$tabbar_item	= wpjam_get_tabbar('weapp', $page_key);

			if(!empty($tabbar_item['webview'])){
				$fields['url']	= ['title'=>'链接',	'type'=>'url',	'required'];
			}
		}

		$fields['apply_for']	= ['title'=>'应用于',	'type'=>'view',	'show_admin_column'=>'only'];

		return $fields;
	}
}

if(weapp_get_current_appid()){
	$weapp_appid	= weapp_get_current_appid();
	$option_name	= 'weapp_tabbar_'.$weapp_appid;

	add_filter('option_'.$option_name,	['WEAPP_Tabbar', 'filter_option']);

	wpjam_register_option($option_name, ['model'=>'WEAPP_Tabbar']);
}
	
function weapp_get_tabbar_options(){
	return wp_list_pluck(wpjam_get_tabbar('weapp'), 'text');
}

function weapp_get_tabbar(){
	return WEAPP_Tabbar::get_items();
}

if(is_weapp()){
	wpjam_register_config('tabbar', 'weapp_get_tabbar');
}