<?php
class WEAPP_Subscribe_Message extends WPJAM_Model{
	public static function register_json($json){
		if(wp_is_json_request()){
			$templates	= wpjam_get_parameter('weapp_templates',	['method'=>'POST']);

			if(is_array($templates)){
				self::subscribe($templates);
			}
		}

		if(in_array($json, ['templates.subscribe', 'weapp.templates.subscribe'])){
			return wpjam_register_api($json, ['callback'=>[self::class, 'api_subscribe']]);
		}
	}

	public static function api_subscribe(){
		$templates	= wpjam_get_parameter('templates',	['method'=>'POST',	'required'=>true]);
		$result		= self::subscribe($templates);

		if(is_wp_error($result)){
			return $result;
		}

		return [];
	}

	public static function subscribe($templates){
		$openid	= weapp_get_current_openid();

		if(is_wp_error($openid)){
			return $openid;
		}

		$account	= weapp_get_user_object($openid);

		foreach($templates as $template){
			$account->subscribe($template['template_id']);
		}
	}

	public static function get_sub_pages($base_menu, $appid){
		return [$base_menu.'-templates'	=> [
			'menu_title'	=> '订阅消息模板',	
			'function'		=> 'tab',
			'load_callback'	=> [self::class, 'page_load']
		]];
	}

	public static function page_load(){
		wpjam_register_plugin_page_tab('mine', [
			'title'			=> '我的模板',
			'list_type'		=> 'mine',
			'function'		=> 'list',
			'singular'		=> 'weapp-template',
			'plural'		=> 'weapp-templates',
			'primary_key'	=> 'priTmplId',
			'model'			=> 'WEAPP_Subscribe_Message_Template'
		]);

		wpjam_register_plugin_page_tab('pub', [
			'title'			=> '公共模板',
			'list_type'		=> 'pub',
			'function'		=> 'list',
			'singular'		=> 'weapp-template',
			'plural'		=> 'weapp-templates',
			'primary_key'	=> 'tid',
			'per_page'		=> 30,
			'model'			=> 'WEAPP_Subscribe_Message_Template'
		]);

		add_action('admin_head', function(){
			?>
			<script type="text/javascript">
			jQuery(function($){
				$('body').on('list_table_action_success', function(event, response){
					if(response.list_action == 'choose'){
						$("body #tr_keywords .mu-item").last().remove();
					}
				});
			});
			</script>
			<?php
		});
	}

	public static function get_handler(){
		return wpjam_get_handler([
			'table_name'		=> self::get_table(),
			'primary_key'		=> 'id',
			'cache'				=> false,
			'searchable_fields'	=> [],
			'filterable_fields'	=> [],
		]);
	}

	public static function get_table(){
		return $GLOBALS['wpdb']->base_prefix.'weapp_subscribe_message_relationships';
	}

	public static function create_table(){
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

		$table	= self::get_table();

		if($GLOBALS['wpdb']->get_var("show tables like '{$table}'") != $table) {
			$sql = "
			CREATE TABLE IF NOT EXISTS `{$table}` (
				`id` bigint(20) NOT NULL auto_increment,

				`appid` VARCHAR(32) NOT  NULL,
				`openid` VARCHAR(30) NOT  NULL,
				`template_id` varchar(43) NOT NULL,
				`times` int(7) NOT NULL,

				PRIMARY KEY	(`id`),
				KEY `appid_idx` (`appid`),
				KEY `openid_idx` (`openid`)
			) DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;
			";

			dbDelta($sql);
		}
	}

	public static function sanitize($str, $pattern, $length=20){
		if(preg_match_all($pattern, $str, $result)){
			return mb_substr(join('', $result[0]), 0, $length, 'utf-8');
		}else{
			return '';
		}
	}

	public static function sanitize_thing($str){
		// 20个以内字符	可汉字、数字、字母或符号组合(这条正则不匹配符号)
		return self::sanitize($str, '/[\x{4e00}-\x{9fa5}a-zA-Z0-9]{1,20}/u',20);
	}

	public static function sanitize_number($str){
		//32位以内数字	只能数字，可带小数 
		//该正则可能需要优化 该条正则会使全数字时取出30位 带小数取出32位
		return self::sanitize($str, '/([0-9](\.[0-9])?){1,30}/u',30);
	}

	public static function sanitize_character_string($str){
		//32位以内数字、字母或符号	可数字、字母或符号组合 但符号只匹配%&,;
		return self::sanitize($str, '/[%&,;a-zA-Z0-9]{1,32}/u',32);
	}

	public static function sanitize_phrase($str){
		//5个以内纯汉字，例如：配送中
		return self::sanitize($str, '/[\x{4e00}-\x{9fa5}]{1,5}/u',5);
	}

	public static function sanitize_amount($str){
		//1个币种符号+10位以内纯数字，可带小数，结尾可带“元”
		//以$或￥开头的后取10位长度 纯数字只获取8位长度
		return self::sanitize($str, '/(\$|\￥)?([0-9](\.[0-9]{0,2})?){1,8}(元)?/u',8);
	}
}

class WEAPP_Subscribe_Message_Template{
	public static function get_items($appid=''){
		return weapp_get_setting('subscribe_message_templates', $appid) ?: [];
	}

	public static function update_items($items, $appid=''){
		return weapp_update_setting('subscribe_message_templates', $items, $appid);
	}

	public static function get_item($key, $appid=''){
		$items	= self::get_items($appid);

		return $items[$key] ?? false;
	}

	public static function parse_items($appid=''){
		$templates	= [];
	
		foreach(self::get_items($appid) as $key => $template_id){
			$templates[]	= compact('key', 'template_id');
		}

		return $templates;
	}

	public static function get_config(){
		return ['subscribe_message_templates'=>self::parse_items()];
	}

	public static function get_list_type(){
		return $GLOBALS['wpjam_list_table']->list_type;
	}

	public static function generating($value=null){
		$key		= 'weapp_subscribe_message_templates_generating';
		$current	= get_transient($key) ?: 0;

		if(!is_null($value) && $current != $value){
			if($value){
				set_transient($key, $value, 100);
			}else{
				delete_transient($key);
			}
		}

		return $current;
	}

	public static function generate(){
		if(self::generating(1)){
			return;
		}

		$template_objects = weapp_get_subscribe_message_template_objects();

		if(empty($template_objects)){
			return;
		}

		$has_templates	= weapp_get_template();

		if(is_wp_error($has_templates)){
			return $has_templates;
		}

		$category	= weapp_get_template_category();

		if(is_wp_error($category)){
			return $category;
		}

		$has_templates	= wp_list_pluck($has_templates, 'priTmplId', 'title');
		$category_ids	= wp_list_pluck($category, 'id');

		$template_ids	= [];

		foreach($template_objects as $k => $object) {
			if($object->category_id && !in_array($object->category_id, $category_ids)){
				continue;
			}

			$title	= $object->title;

			if($has_templates && isset($has_templates[$title])){
				$template_ids[$k]	= $has_templates[$title];
			}else{
				// $template_library  = weapp_get_pub_template_keywords($object->tid);

				// if(is_wp_error($template_library)){
				// 	continue;
				// }

				// if($template_library['title'] == $object->title){
					$scene_desc		= $object->sceneDesc ?? $object->title;
					$template_id	= weapp_add_template($object->tid, $object->kidList, $scene_desc);

					if(!is_wp_error($template_id)){
						$template_ids[$k]	= $template_id;
					}
				// }
			}
		}

		self::update_items($template_ids);

		return $template_ids;
	}

	public static function get($tid){
		if(self::get_list_type() == 'pub'){
			$keywords	= weapp_get_pub_template_keywords($tid);

			return compact('tid','keywords');
		}
	}

	public static function choose($tid, $data){
		self::generating(0);

		$template_id	= weapp_add_template($tid,  array_column($data['keywords'], 'kid'), $data['sceneDesc']);

		return is_wp_error($template_id) ? $template_id : admin_url('page=weapp-templates&id='.$template_id);
	}

	public static function delete($priTmplId){
		self::generating(0);

		return weapp_del_template($priTmplId);
	}

	public static function query_items($count, $offset){
		if(self::get_list_type() == 'pub'){
			$result = weapp_get_pub_template_titles($offset, $count);

			if(is_wp_error($result)){
				return $result;
			}

			$items	= $result['data'];
			$total	= $result['count'];
		}else{
			self::generate();

			$items	= weapp_get_template();

			if(is_wp_error($items)){
				return $items;
			}

			$total	= count($items);
		}

		return compact('items', 'total');
	}

	public static function render_item($item){
		if(self::get_list_type() == 'mine'){
			$template_ids		= weapp_get_setting('subscribe_message_templates') ?: [];

			if($template_ids){
				$template_keys	= array_flip($template_ids);
				$template_key	= $template_keys[$item['priTmplId']] ?? '';

				$item['priTmplId']	= '模板&ensp;ID：'.$item['priTmplId'].= '<br />调用Key：'.$template_key;
			}

			$template_types		= [2 =>'一次性订阅', 3=>'长期订阅'];

			$item['title']		= '标题：'.$item['title'].'<br />类型：'.$template_types[$item['type']];

			$item['content']	= wpautop($item['content']);
			$item['example']	= wpautop($item['example']);
		}

		return $item;
	}

	Public static function get_actions(){
		if(self::get_list_type() == 'pub'){
			return ['choose'=>['title'=>'选用',	'response'=>'redirect']];
		}else{
			return ['delete'=>['title'=>'删除',	'direct'=>true,	'confirm'=>true,	'bulk'=>true]];
		}
	}

	Public static function get_fields($action_key='', $tid=''){
		if(self::get_list_type() == 'pub'){
			if($action_key == ''){
				$cat_options	= [];

				foreach(weapp_get_template_category() as $category){
					$cat_options[$category['id']]	= $category['name'].'（'.$category['id'].'）';
				}

				return [
					'title'			=> ['title'=>'模板标题',		'type'=>'view',	'show_admin_column'=>'only'],
					'tid'			=> ['title'=>'模板ID',		'type'=>'view',	'show_admin_column'=>true],
					'categoryId'	=> ['title'=>'模板分类',		'type'=>'view',	'show_admin_column'=>'only',	'options'=>$cat_options],
					'type'			=> ['title'=>'模板类型',		'type'=>'view',	'show_admin_column'=>'only',	'options'=>[2 =>'一次性订阅', 3=>'长期订阅']],
				];
			}else{
				return [
					'tid'		=> ['title'=>'模板ID',		'type'=>'view',	'show_admin_column'=>true],
					'sceneDesc'	=> ['title'=>'使用场景',		'type'=>'text',	'placeholder'=>'请输入15字的输入场景说明'],
					'keywords'	=> ['title'=>'关键词库',		'type'=>'mu-fields',	'fields'=>[
						'kid'		=> ['title'=>'',	'type'=>'hidden'],
						'name'		=> ['title'=>'关键词内容：',	'type'=>'view'],
						'example'	=> ['title'=>'对应的示例：',	'type'=>'view'],
						// 'rule'		=> ['title'=>'参数类型',		'type'=>'view'],
					]],
				];
			}
		}else{
			return [
				'title'		=> ['title'=>'模板标题',		'type'=>'text',	'show_admin_column'=>true],
				'content'	=> ['title'=>'模板内容',		'type'=>'text',	'show_admin_column'=>true],
				'example'	=> ['title'=>'模板内容示例',	'type'=>'text',	'show_admin_column'=>true],
				'priTmplId'	=> ['title'=>'模板ID',		'type'=>'text',	'show_admin_column'=>true]
			];
		}
	}
}

if(is_weapp()){
	wpjam_register_config(['WEAPP_Subscribe_Message_Template', 'get_config']);
}

function weapp_register_subscribe_message_template($key, $args){
	return wpjam_register('weapp_subscribe_message_template', $key, $args);
}

function weapp_get_subscribe_message_template_objects(){
	return wpjam_get_registereds('weapp_subscribe_message_template');
}

function weapp_get_subscribe_message_template_id($key, $appid=''){
	return WEAPP_Subscribe_Message_Template::get_item($key, $appid);
}

function weapp_get_subscribe_message_templates($appid=''){
	return WEAPP_Subscribe_Message_Template::parse_items($appid);
}

function weapp_get_template($appid=''){
	$response	= weapp($appid)->get('/wxaapi/newtmpl/gettemplate');

	return is_wp_error($response) ? $response : $response['data'];
}

function weapp_get_pub_template_titles($start=0, $limit=20, $appid=''){
	$category	= weapp_get_template_category($appid);

	if(is_wp_error($category)){
		return $category;
	}

	$ids	= implode(',', array_column($category, 'id'));

	return weapp($appid)->get('/wxaapi/newtmpl/getpubtemplatetitles?ids='.$ids.'&start='.$start.'&limit='.$limit);
}

function weapp_get_template_category($appid=''){
	$cache_key	= 'weapp_template_category_'.weapp_get_current_appid();
	$category	= wp_cache_get($cache_key, 'counts');

	if($category === false){
		$response	= weapp($appid)->get('/wxaapi/newtmpl/getcategory');

		if(is_wp_error($response)){
			return $response;
		}

		$category	= $response['data'];

		wp_cache_set($cache_key, $category, 'counts');
	}

	return $category;
}

function weapp_get_pub_template_keywords($tid, $appid=''){
	$response	= weapp($appid)->get('/wxaapi/newtmpl/getpubtemplatekeywords', compact('tid'));

	return is_wp_error($response) ? $response : $response['data'];
}

function weapp_add_template($tid, $kidList, $sceneDesc='', $appid=''){
	$tid		= (string)$tid;
	$response	= weapp($appid)->post('/wxaapi/newtmpl/addtemplate', compact('tid', 'kidList', 'sceneDesc'));

	return is_wp_error($response) ? $response : $response['priTmplId'];
}

function weapp_del_template($priTmplId, $appid=''){
	return weapp($appid)->post('/wxaapi/newtmpl/deltemplate', compact('priTmplId'));
}

function weapp_send_subscribe_message($data, $appid=''){
	$appid	= $appid ?: weapp_get_current_appid();
	$openid	= $data['touser'];

	if($appid && $openid){
		$user	= weapp_get_user_object($openid, $appid);

		if($user->consume($data['template_id'])){
			return weapp($appid)->post('/cgi-bin/message/subscribe/send', $data);
		}
	}
	
	return false;
}


class_alias('WEAPP_Subscribe_Message', 'WEAPP_SubscribeMessage');